/*****************************************************************************************************************************
--
--    File Name    : Bilinear_Interpolation.v 

--    Description  : This module implements Bilinear Interpolation on a 3x3 window.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module Bilinear_Interpolation #(parameter DATA_WIDTH = 8)
                              (input wire[1 : 0] op_sel,
                               input wire[DATA_WIDTH - 1 : 0] p11_i,
                               input wire[DATA_WIDTH - 1 : 0] p12_i,
                               input wire[DATA_WIDTH - 1 : 0] p13_i,
                               input wire[DATA_WIDTH - 1 : 0] p21_i,
                               input wire[DATA_WIDTH - 1 : 0] p22_i,
                               input wire[DATA_WIDTH - 1 : 0] p23_i,
                               input wire[DATA_WIDTH - 1 : 0] p31_i,
                               input wire[DATA_WIDTH - 1 : 0] p32_i,
                               input wire[DATA_WIDTH - 1 : 0] p33_i,
                               output reg[DATA_WIDTH + 1 : 0] R_Out_o,
                               output reg[DATA_WIDTH + 1 : 0] G_Out_o,
                               output reg[DATA_WIDTH + 1 : 0] B_Out_o);
  reg signed[DATA_WIDTH + 1 : 0] lI;
  reg signed[DATA_WIDTH + 1 : 0] oI;
  reg signed[DATA_WIDTH + 1 : 0] iI;
  always
    @(*)
    begin
      case (op_sel)
        2'd0:
          begin
            lI = (p11_i + p13_i + p31_i + p33_i);
            oI = (p12_i + p32_i + p21_i + p23_i);
            iI = p22_i;
          end
        2'd1:
          begin
            lI = (p12_i + p32_i);
            oI = p22_i;
            iI = (p21_i + p23_i);
          end
        2'd2:
          begin
            lI = (p21_i + p23_i);
            oI = p22_i;
            iI = (p12_i + p32_i);
          end
        2'd3:
          begin
            lI = p22_i;
            oI = (p12_i + p21_i + p23_i + p32_i);
            iI = (p11_i + p13_i + p31_i + p33_i);
          end
        default:
          ;
      endcase
    end
  always
    @(*)
    begin
      case (op_sel)
        2'd0:
          begin
            R_Out_o = { 2'b00, lI[DATA_WIDTH + 1 : 2] };
            G_Out_o = { 2'b00, oI[DATA_WIDTH + 1 : 2] };
            B_Out_o = iI[DATA_WIDTH + 1 : 0];
          end
        2'd1:
          begin
            R_Out_o = { 1'b0, lI[DATA_WIDTH + 1 : 1] };
            G_Out_o = oI[DATA_WIDTH + 1 : 0];
            B_Out_o = { 1'b0, iI[DATA_WIDTH + 1 : 1] };
          end
        2'd2:
          begin
            R_Out_o = { 1'b0, lI[DATA_WIDTH + 1 : 1] };
            G_Out_o = oI[DATA_WIDTH + 1 : 0];
            B_Out_o = { 1'b0, iI[DATA_WIDTH + 1 : 1] };
          end
        2'd3:
          begin
            R_Out_o = lI[DATA_WIDTH + 1 : 0];
            G_Out_o = { 2'b00, oI[DATA_WIDTH + 1 : 2] };
            B_Out_o = { 2'b00, iI[DATA_WIDTH + 1 : 2] };
          end
        default:
          ;
      endcase
    end
endmodule
