/*****************************************************************************************************************************
--
--    File Name    : Median.v 

--    Description  : This module implements Median of 3x3 window.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module Median #(parameter g_DATAWIDTH = 8)
              (input wire SYS_CLK_I,
               input wire RESET_n_I,
               input wire[g_DATAWIDTH - 1 : 0] A11_i,
               input wire[g_DATAWIDTH - 1 : 0] A12_i,
               input wire[g_DATAWIDTH - 1 : 0] A13_i,
               input wire[g_DATAWIDTH - 1 : 0] A21_i,
               input wire[g_DATAWIDTH - 1 : 0] A22_i,
               input wire[g_DATAWIDTH - 1 : 0] A23_i,
               input wire[g_DATAWIDTH - 1 : 0] A31_i,
               input wire[g_DATAWIDTH - 1 : 0] A32_i,
               input wire[g_DATAWIDTH - 1 : 0] A33_i,
               output wire[g_DATAWIDTH - 1 : 0] Median);
  wire[g_DATAWIDTH - 1 : 0] iI0;
  wire[g_DATAWIDTH - 1 : 0] Ol0;
  wire[g_DATAWIDTH - 1 : 0] Il0;
  wire[g_DATAWIDTH - 1 : 0] ll0;
  wire[g_DATAWIDTH - 1 : 0] ol0;
  wire[g_DATAWIDTH - 1 : 0] il0;
  wire[g_DATAWIDTH - 1 : 0] O00;
  wire[g_DATAWIDTH - 1 : 0] I00;
  wire[g_DATAWIDTH - 1 : 0] l00;
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) o00(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A11_i),
                                                 .B(A12_i),
                                                 .C(A13_i),
                                                 .M(iI0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) i00(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A21_i),
                                                 .B(A22_i),
                                                 .C(A23_i),
                                                 .M(Ol0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) O10(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A31_i),
                                                 .B(A32_i),
                                                 .C(A33_i),
                                                 .M(Il0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) I10(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A11_i),
                                                 .B(A21_i),
                                                 .C(A31_i),
                                                 .M(ll0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) l10(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A12_i),
                                                 .B(A22_i),
                                                 .C(A32_i),
                                                 .M(ol0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) o10(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A13_i),
                                                 .B(A23_i),
                                                 .C(A33_i),
                                                 .M(il0));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) i10(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(A11_i),
                                                 .B(A22_i),
                                                 .C(A33_i),
                                                 .M(O00));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) Oo0(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(iI0),
                                                 .B(Ol0),
                                                 .C(Il0),
                                                 .M(I00));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) Io0(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(ll0),
                                                 .B(ol0),
                                                 .C(il0),
                                                 .M(l00));
  Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH)) lo0(.SYS_CLK_I(SYS_CLK_I),
                                                 .RESET_n_I(RESET_n_I),
                                                 .A(I00),
                                                 .B(l00),
                                                 .C(O00),
                                                 .M(Median));
endmodule
