/*****************************************************************************************************************************
--
--    File Name    : Image_Sharpen_Filter.v 

--    Description  : This module implements Sharpen Filter .

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module ImageSharpenFilter #(parameter g_DATAWIDTH = 24,
                            parameter g_X_RES_WIDTH = 11,
                            parameter g_DISPLAY_RESOLUTION = 1280,
                            parameter g_VERT_DISPLAY_RESOLUTION = 720)
                          (input wire SYS_CLK_I,
                           input wire RESET_n_I,
                           input wire[(g_DATAWIDTH / 3) - 1 : 0] Y_in_i,
                           input wire[(g_DATAWIDTH / 3) - 1 : 0] Cb_in_i,
                           input wire[(g_DATAWIDTH / 3) - 1 : 0] Cr_in_i,
                           input wire Data_In_Vld_i,
                           input wire[7 : 0] sharpness_K,
                           output reg[(g_DATAWIDTH / 3) - 1 : 0] Y_Out_o,
                           output reg[(g_DATAWIDTH / 3) - 1 : 0] Cb_Out_o,
                           output reg[(g_DATAWIDTH / 3) - 1 : 0] Cr_Out_o,
                           output reg data_Vld_out_o);
  parameter[0 : 0] O = 1'b0,
                   I = 1'b1;
  parameter[2 : 0] l = 3'h0,
                   o = 3'h1,
                   i = 3'h2,
                   OI = 3'h3,
                   II = 3'h4,
                   lI = 3'h5;
  parameter oI = 8;
  parameter iI = 6;
  reg[g_X_RES_WIDTH - 1 : 0] Ol;
  reg[g_DATAWIDTH - 1 : 0] Il;
  reg ll;
  reg[2 : 0] ol;
  reg[g_X_RES_WIDTH - 1 : 0] il;
  reg O0;
  reg I0;
  reg l0;
  reg o0;
  reg i0;
  reg O1;
  reg I1;
  reg l1;
  reg o1;
  reg i1;
  reg Oo;
  reg[g_X_RES_WIDTH - 1 : 0] Io;
  reg[g_X_RES_WIDTH - 1 : 0] lo;
  reg[g_X_RES_WIDTH - 1 : 0] oo;
  reg[g_X_RES_WIDTH - 1 : 0] io;
  reg[g_X_RES_WIDTH - 1 : 0] Oi;
  reg[g_X_RES_WIDTH - 1 : 0] Ii;
  reg[g_X_RES_WIDTH - 1 : 0] li;
  reg[g_X_RES_WIDTH - 1 : 0] oi;
  reg[g_X_RES_WIDTH - 1 : 0] ii;
  reg[g_X_RES_WIDTH - 1 : 0] OOI;
  reg[g_DATAWIDTH - 1 : 0] IOI;
  reg[g_DATAWIDTH - 1 : 0] lOI;
  reg[g_DATAWIDTH - 1 : 0] oOI;
  reg[g_DATAWIDTH - 1 : 0] iOI;
  reg[g_DATAWIDTH - 1 : 0] OII;
  reg[g_DATAWIDTH - 1 : 0] III;
  reg[g_DATAWIDTH - 1 : 0] lII;
  reg[g_DATAWIDTH - 1 : 0] oII;
  reg[g_DATAWIDTH - 1 : 0] iII;
  reg[g_DATAWIDTH - 1 : 0] OlI;
  reg[g_DATAWIDTH - 1 : 0] IlI;
  reg[g_DATAWIDTH - 1 : 0] llI;
  reg[(g_DATAWIDTH / 3) - 1 : 0] olI;
  reg[(g_DATAWIDTH / 3) - 1 : 0] ilI;
  reg[(g_DATAWIDTH / 3) - 1 : 0] O0I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] I0I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] l0I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] o0I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] i0I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] O1I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] I1I;
  reg[(g_DATAWIDTH / 3) - 1 : 0] l1I;
  reg[g_X_RES_WIDTH - 1 : 0] o1I;
  reg[g_X_RES_WIDTH - 1 : 0] i1I;
  reg[g_DATAWIDTH - 1 : 0] OoI;
  reg[g_DATAWIDTH - 1 : 0] IoI;
  reg[g_DATAWIDTH - 1 : 0] loI;
  reg[g_DATAWIDTH - 1 : 0] ooI;
  reg[g_DATAWIDTH - 1 : 0] ioI;
  reg[2 : 0] OiI;
  reg[2 : 0] IiI;
  reg[(g_DATAWIDTH / 3) - 1 : 0] liI[(oI - 1) : 0];
  reg[(g_DATAWIDTH / 3) - 1 : 0] oiI[(oI - 1) : 0];
  reg iiI;
  reg[g_X_RES_WIDTH - 1 : 0] OOl;
  reg[0 : 0] IOl;
  reg[(g_DATAWIDTH / 3) - 1 : 0] lOl[(iI - 1) : 0];
  reg oOl;
  reg iOl;
  reg OIl;
  reg signed[(g_DATAWIDTH / 3 + 7 + 2) : 0] IIl;
  reg signed[(g_DATAWIDTH / 3 + 3) : 0] lIl;
  reg signed[(g_DATAWIDTH / 3 + 4) : 0] oIl;
  reg[2 : 0] iIl;
  reg Oll;
  reg Ill;
  reg lll;
  wire[(g_DATAWIDTH / 3) - 1 : 0] oll;
  wire[g_DATAWIDTH - 1 : 0] ill;
  wire[g_DATAWIDTH - 1 : 0] O0l;
  wire[g_DATAWIDTH - 1 : 0] I0l;
  wire[g_DATAWIDTH - 1 : 0] l0l;
  wire[g_DATAWIDTH - 1 : 0] o0l;
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Ol <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (Ol == g_DISPLAY_RESOLUTION - 1)
            begin
              Ol <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (ll == 1'b1)
                begin
                  Ol <= Ol + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ol <= 3'b000;
        end
      else
        begin
          if ((ol == 3'b100 && Ol == g_DISPLAY_RESOLUTION - 1) || (il == g_VERT_DISPLAY_RESOLUTION - 1 && Ol == g_DISPLAY_RESOLUTION - 1))
            begin
              ol <= 3'b000;
            end
          else
            begin
              if (Ol == g_DISPLAY_RESOLUTION - 1)
                begin
                  ol <= ol + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          il <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (il == g_VERT_DISPLAY_RESOLUTION)
            begin
              il <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (Ol == g_DISPLAY_RESOLUTION - 1 && ll == 1'b1)
                begin
                  il <= il + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Ill <= 1'b0;
        end
      else
        begin
          if (il == g_VERT_DISPLAY_RESOLUTION)
            begin
              Ill <= 1'b1;
            end
          else
            if (lll)
              begin
                Ill <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Il <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          Il <= { Y_in_i, Cb_in_i, Cr_in_i };
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ll <= 1'b0;
        end
      else
        begin
          ll <= Data_In_Vld_i;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          I0 <= 1'b0;
          l0 <= 1'b0;
          o0 <= 1'b0;
          i0 <= 1'b0;
          O1 <= 1'b0;
        end
      else
        if (Data_In_Vld_i)
          begin
            case (ol)
              3'd0:
                begin
                  I0 <= 1'b1;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                  i0 <= 1'b0;
                  O1 <= 1'b0;
                end
              3'd1:
                begin
                  I0 <= 1'b0;
                  l0 <= 1'b1;
                  o0 <= 1'b0;
                  i0 <= 1'b0;
                  O1 <= 1'b0;
                end
              3'd2:
                begin
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b1;
                  i0 <= 1'b0;
                  O1 <= 1'b0;
                end
              3'd3:
                begin
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                  i0 <= 1'b1;
                  O1 <= 1'b0;
                end
              3'd4:
                begin
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                  i0 <= 1'b0;
                  O1 <= 1'b1;
                end
              default:
                begin
                  I0 <= 1'b0;
                  l0 <= 1'b0;
                  o0 <= 1'b0;
                  i0 <= 1'b0;
                  O1 <= 1'b0;
                end
            endcase
          end
        else
          begin
            I0 <= 1'b0;
            l0 <= 1'b0;
            o0 <= 1'b0;
            i0 <= 1'b0;
            O1 <= 1'b0;
          end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          O0 <= 1'b0;
        end
      else
        begin
          if ((ol >= 3'b010) && (Ol >= 11'h004))
            begin
              O0 <= 1'b1;
            end
          else
            if ((i1I == g_VERT_DISPLAY_RESOLUTION - 1) && (o1I == g_DISPLAY_RESOLUTION - 1))
              begin
                O0 <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iIl <= l;
        end
      else
        begin
          case (iIl)
            l:
              begin
                if (ol >= 3'b010 && Ol >= 11'h004)
                  begin
                    iIl <= o;
                  end
                else
                  begin
                    iIl <= l;
                  end
              end
            o:
              begin
                if (o1I == g_DISPLAY_RESOLUTION)
                  begin
                    iIl <= lI;
                  end
                else
                  begin
                    iIl <= o;
                  end
              end
            lI:
              begin
                if (Ill)
                  begin
                    iIl <= OI;
                  end
                else
                  begin
                    iIl <= i;
                  end
              end
            i:
              begin
                if (Data_In_Vld_i && Ol >= 11'h004 || (il == 0))
                  begin
                    iIl <= OI;
                  end
                else
                  begin
                    iIl <= i;
                  end
              end
            OI:
              begin
                if (i1I == g_VERT_DISPLAY_RESOLUTION - 1)
                  begin
                    iIl <= II;
                  end
                else
                  begin
                    iIl <= o;
                  end
              end
            II:
              begin
                if (o1I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    iIl <= l;
                  end
                else
                  begin
                    iIl <= II;
                  end
              end
            default:
              iIl <= l;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Oll <= 1'b0;
          lll <= 1'b0;
        end
      else
        begin
          case (iIl)
            l:
              begin
                Oll <= 1'b0;
                lll <= 1'b0;
              end
            o:
              begin
                if (o1I == g_DISPLAY_RESOLUTION)
                  begin
                    Oll <= 1'b1;
                  end
                else
                  begin
                    Oll <= 1'b0;
                  end
              end
            lI:
              begin
                Oll <= 1'b1;
              end
            i:
              begin
                if (Data_In_Vld_i && Ol >= 11'h004 || (il == 0))
                  begin
                    Oll <= 1'b0;
                  end
                else
                  begin
                    Oll <= Oll;
                  end
              end
            OI:
              begin
                Oll <= 1'b0;
              end
            II:
              begin
                if (o1I == g_DISPLAY_RESOLUTION - 1)
                  begin
                    Oll <= 1'b0;
                    lll <= 1'b1;
                  end
                else
                  begin
                    Oll <= Oll;
                    lll <= 1'b0;
                  end
              end
            default:
              begin
                Oll <= 1'b0;
                lll <= 1'b0;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          o1I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (O0 == 1'b0 || o1I > g_DISPLAY_RESOLUTION - 1)
            begin
              o1I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            if (O0 == 1'b1)
              begin
                if (Oll)
                  o1I <= { g_X_RES_WIDTH { 1'b0 } };
                else
                  o1I <= o1I + 1'b1;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          i1I <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (O0 == 1'b0)
            begin
              i1I <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (o1I > g_DISPLAY_RESOLUTION - 1 && O0 == 1'b1)
                begin
                  i1I <= i1I + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OiI <= 3'd0;
        end
      else
        begin
          if (O0 == 1'b0 || (OiI == 3'd4 && o1I == g_DISPLAY_RESOLUTION))
            begin
              OiI <= 3'd0;
            end
          else
            begin
              if (o1I >= g_DISPLAY_RESOLUTION && O0 == 1'b1)
                begin
                  OiI <= OiI + 12'd1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IiI <= 3'd0;
        end
      else
        begin
          IiI <= OiI;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          I1 <= 1'b1;
          l1 <= 1'b1;
          o1 <= 1'b1;
          i1 <= 1'b1;
          Oo <= 1'b1;
        end
      else
        begin
          case (IiI)
            3'd0:
              begin
                I1 <= 1'b0;
                l1 <= 1'b0;
                o1 <= 1'b0;
                i1 <= 1'b1;
                Oo <= 1'b1;
              end
            3'd1:
              begin
                I1 <= 1'b1;
                l1 <= 1'b0;
                o1 <= 1'b0;
                i1 <= 1'b0;
                Oo <= 1'b1;
              end
            3'd2:
              begin
                I1 <= 1'b1;
                l1 <= 1'b1;
                o1 <= 1'b0;
                i1 <= 1'b0;
                Oo <= 1'b0;
              end
            3'd3:
              begin
                I1 <= 1'b0;
                l1 <= 1'b1;
                o1 <= 1'b1;
                i1 <= 1'b0;
                Oo <= 1'b0;
              end
            3'd4:
              begin
                I1 <= 1'b0;
                l1 <= 1'b0;
                o1 <= 1'b1;
                i1 <= 1'b1;
                Oo <= 1'b0;
              end
            default:
              begin
                I1 <= 1'b0;
                l1 <= 1'b0;
                o1 <= 1'b0;
                i1 <= 1'b0;
                Oo <= 1'b0;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Io <= { g_X_RES_WIDTH { 1'b0 } };
          lo <= { g_X_RES_WIDTH { 1'b0 } };
          oo <= { g_X_RES_WIDTH { 1'b0 } };
          io <= { g_X_RES_WIDTH { 1'b0 } };
          Oi <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if ((O0 == 1'b1 && (Io > g_DISPLAY_RESOLUTION - 1 || lo > g_DISPLAY_RESOLUTION - 1 || oo > g_DISPLAY_RESOLUTION - 1 || io > g_DISPLAY_RESOLUTION - 1 || Oi > g_DISPLAY_RESOLUTION - 1)))
            begin
              Io <= { g_X_RES_WIDTH { 1'b0 } };
              lo <= { g_X_RES_WIDTH { 1'b0 } };
              oo <= { g_X_RES_WIDTH { 1'b0 } };
              io <= { g_X_RES_WIDTH { 1'b0 } };
              Oi <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (O0 == 1'b1)
                begin
                  if (Oll == 1'b1)
                    begin
                      Io <= Io;
                      lo <= lo;
                      oo <= oo;
                      io <= io;
                      Oi <= Oi;
                    end
                  else
                    begin
                      case (IiI)
                        3'd0:
                          begin
                            Io <= Io + 1'b1;
                            lo <= lo + 1'b1;
                            oo <= oo + 1'b1;
                          end
                        3'd1:
                          begin
                            lo <= lo + 1'b1;
                            oo <= oo + 1'b1;
                            io <= io + 1'b1;
                          end
                        3'd2:
                          begin
                            oo <= oo + 1'b1;
                            io <= io + 1'b1;
                            Oi <= Oi + 1'b1;
                          end
                        3'd3:
                          begin
                            io <= io + 1'b1;
                            Oi <= Oi + 1'b1;
                            Io <= Io + 1'b1;
                          end
                        3'd4:
                          begin
                            Oi <= Oi + 1'b1;
                            Io <= Io + 1'b1;
                            lo <= lo + 1'b1;
                          end
                        default:
                          begin
                            Io <= Io;
                            lo <= lo;
                            oo <= oo;
                            io <= io;
                            Oi <= Oi;
                          end
                      endcase
                    end
                end
              else
                if (O0 == 1'b0)
                  begin
                    Io <= { g_X_RES_WIDTH { 1'b0 } };
                    lo <= { g_X_RES_WIDTH { 1'b0 } };
                    oo <= { g_X_RES_WIDTH { 1'b0 } };
                    io <= { g_X_RES_WIDTH { 1'b0 } };
                    Oi <= { g_X_RES_WIDTH { 1'b0 } };
                  end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Ii <= { g_X_RES_WIDTH { 1'b0 } };
          li <= { g_X_RES_WIDTH { 1'b0 } };
          oi <= { g_X_RES_WIDTH { 1'b0 } };
          ii <= { g_X_RES_WIDTH { 1'b0 } };
          OOI <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          Ii <= Io;
          li <= lo;
          oi <= oo;
          ii <= io;
          OOI <= Oi;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OoI <= { g_DATAWIDTH { 1'b0 } };
          IoI <= { g_DATAWIDTH { 1'b0 } };
          loI <= { g_DATAWIDTH { 1'b0 } };
          ooI <= { g_DATAWIDTH { 1'b0 } };
          ioI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          OoI <= ill;
          IoI <= O0l;
          loI <= I0l;
          ooI <= l0l;
          ioI <= o0l;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IOI <= { g_DATAWIDTH { 1'b0 } };
          OII <= { g_DATAWIDTH { 1'b0 } };
          iII <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (oi == 11'd1 || ii == 11'd1 || OOI == 11'd1)
            begin
              case (IiI)
                3'd0:
                  begin
                    IOI <= OoI;
                    OII <= IoI;
                    iII <= loI;
                  end
                3'd1:
                  begin
                    IOI <= IoI;
                    OII <= loI;
                    iII <= ooI;
                  end
                3'd2:
                  begin
                    IOI <= loI;
                    OII <= ooI;
                    iII <= ioI;
                  end
                3'd3:
                  begin
                    IOI <= ooI;
                    OII <= ioI;
                    iII <= OoI;
                  end
                3'd4:
                  begin
                    IOI <= ioI;
                    OII <= OoI;
                    iII <= IoI;
                  end
                default:
                  begin
                    IOI <= IOI;
                    OII <= OII;
                    iII <= iII;
                  end
              endcase
            end
          else
            begin
              if (oi >= 11'd5 || ii >= 11'd5 || OOI >= 11'd5)
                begin
                  IOI <= lOI;
                  OII <= III;
                  iII <= OlI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          lOI <= { g_DATAWIDTH { 1'b0 } };
          III <= { g_DATAWIDTH { 1'b0 } };
          OlI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (oi == 11'd2 || ii == 11'd2 || OOI == 11'd2)
            begin
              case (IiI)
                3'd0:
                  begin
                    lOI <= OoI;
                    III <= IoI;
                    OlI <= loI;
                  end
                3'd1:
                  begin
                    lOI <= IoI;
                    III <= loI;
                    OlI <= ooI;
                  end
                3'd2:
                  begin
                    lOI <= loI;
                    III <= ooI;
                    OlI <= ioI;
                  end
                3'd3:
                  begin
                    lOI <= ooI;
                    III <= ioI;
                    OlI <= OoI;
                  end
                3'd4:
                  begin
                    lOI <= ioI;
                    III <= OoI;
                    OlI <= IoI;
                  end
                default:
                  begin
                    lOI <= lOI;
                    III <= III;
                    OlI <= OlI;
                  end
              endcase
            end
          else
            begin
              if (oi >= 11'd5 || ii >= 11'd5 || OOI >= 11'd5)
                begin
                  lOI <= oOI;
                  III <= lII;
                  OlI <= IlI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          oOI <= { g_DATAWIDTH { 1'b0 } };
          lII <= { g_DATAWIDTH { 1'b0 } };
          IlI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (oi == 11'd3 || ii == 11'd3 || OOI == 11'd3)
            begin
              case (IiI)
                3'd0:
                  begin
                    oOI <= OoI;
                    lII <= IoI;
                    IlI <= loI;
                  end
                3'd1:
                  begin
                    oOI <= IoI;
                    lII <= loI;
                    IlI <= ooI;
                  end
                3'd2:
                  begin
                    oOI <= loI;
                    lII <= ooI;
                    IlI <= ioI;
                  end
                3'd3:
                  begin
                    oOI <= ooI;
                    lII <= ioI;
                    IlI <= OoI;
                  end
                3'd4:
                  begin
                    oOI <= ioI;
                    lII <= OoI;
                    IlI <= IoI;
                  end
                default:
                  begin
                    oOI <= oOI;
                    lII <= lII;
                    IlI <= IlI;
                  end
              endcase
            end
          else
            begin
              if (oi >= 11'd5 || ii >= 11'd5 || OOI >= 11'd5)
                begin
                  oOI <= iOI;
                  lII <= oII;
                  IlI <= llI;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iOI <= { g_DATAWIDTH { 1'b0 } };
          oII <= { g_DATAWIDTH { 1'b0 } };
          llI <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (oi >= 11'd4 || ii >= 11'd4 || OOI >= 11'd4)
            begin
              case (IiI)
                3'd0:
                  begin
                    iOI <= OoI;
                    oII <= IoI;
                    llI <= loI;
                  end
                3'd1:
                  begin
                    iOI <= IoI;
                    oII <= loI;
                    llI <= ooI;
                  end
                3'd2:
                  begin
                    iOI <= loI;
                    oII <= ooI;
                    llI <= ioI;
                  end
                3'd3:
                  begin
                    iOI <= ooI;
                    oII <= ioI;
                    llI <= OoI;
                  end
                3'd4:
                  begin
                    iOI <= ioI;
                    oII <= OoI;
                    llI <= IoI;
                  end
                default:
                  begin
                    iOI <= iOI;
                    oII <= oII;
                    llI <= llI;
                  end
              endcase
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iiI <= 1'b0;
        end
      else
        begin
          if (Ii == g_DISPLAY_RESOLUTION || li == g_DISPLAY_RESOLUTION || oi == g_DISPLAY_RESOLUTION || ii == g_DISPLAY_RESOLUTION || OOI == g_DISPLAY_RESOLUTION)
            begin
              iiI <= 1'b1;
            end
          else
            if (Ii <= 11'd4 || li >= 11'd4 || oi >= 11'd4 || ii >= 11'd4 || OOI >= 11'd4)
              begin
                iiI <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          olI <= { (g_DATAWIDTH / 3) { 1'b0 } };
          ilI <= { (g_DATAWIDTH / 3) { 1'b0 } };
          O0I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          I0I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          l0I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          o0I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          i0I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          O1I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          I1I <= { (g_DATAWIDTH / 3) { 1'b0 } };
          liI[0] <= 8'd0;
          oiI[0] <= 8'd0;
        end
      else
        begin
          if (oi >= 11'd5 || ii >= 11'd5 || OOI >= 11'd5 || li >= 11'd5 || Ii >= 11'd5 || iiI)
            begin
              olI <= IOI[23 : 16];
              ilI <= lOI[23 : 16];
              O0I <= oOI[23 : 16];
              I0I <= OII[23 : 16];
              l0I <= III[23 : 16];
              o0I <= lII[23 : 16];
              i0I <= iII[23 : 16];
              O1I <= OlI[23 : 16];
              I1I <= IlI[23 : 16];
              liI[0] <= III[15 : 8];
              oiI[0] <= III[7 : 0];
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IOl <= O;
          OOl <= 11'd0;
          oOl <= 1'b0;
        end
      else
        begin
          case (IOl)
            O:
              begin
                if (o1I == 11'd11)
                  begin
                    OOl <= OOl + 1'b1;
                    oOl <= 1'b1;
                    IOl <= I;
                  end
                else
                  begin
                    IOl <= O;
                    OOl <= 11'd0;
                    oOl <= 1'b0;
                  end
              end
            I:
              begin
                if (OOl == g_DISPLAY_RESOLUTION)
                  begin
                    IOl <= O;
                    OOl <= 11'd0;
                    oOl <= 1'b0;
                  end
                else
                  begin
                    OOl <= OOl + 1'b1;
                    oOl <= 1'b1;
                    IOl <= I;
                  end
              end
            default:
              begin
                IOl <= O;
              end
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin: i0l
      integer O1l;
      if (!RESET_n_I)
        begin
          for (O1l = 1; O1l < oI; O1l = O1l + 1)
            begin
              liI[O1l] <= 8'd0;
              oiI[O1l] <= 8'd0;
            end
        end
      else
        begin
          for (O1l = 1; O1l < oI; O1l = O1l + 1)
            begin
              liI[O1l] <= liI[O1l - 1];
              oiI[O1l] <= oiI[O1l - 1];
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          lOl[0] <= 8'd0;
        end
      else
        begin
          lOl[0] <= l0I;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin: I1l
      integer l1l;
      if (!RESET_n_I)
        begin
          for (l1l = 1; l1l < iI; l1l = l1l + 1)
            begin
              lOl[l1l] <= 8'd0;
            end
        end
      else
        begin
          for (l1l = 1; l1l < iI; l1l = l1l + 1)
            begin
              lOl[l1l] <= lOl[l1l - 1];
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          l1I <= 8'd0;
        end
      else
        begin
          l1I <= oll;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          IIl <= 18'd0;
        end
      else
        begin
          IIl <= $signed({ 1'b0, l1I }) * $signed({ 1'b0, sharpness_K[7 : 0] });
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          lIl <= 12'd0;
          iOl <= 1'b0;
        end
      else
        begin
          if (|IIl[16 : 15] == 1'b1)
            begin
              lIl <= 12'd18;
            end
          else
            begin
              lIl <= IIl[17 : 6];
            end
          iOl <= oOl;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          oIl <= 13'd0;
          OIl <= 1'b0;
        end
      else
        begin
          oIl <= $signed({ 1'b0, lOl[5] }) + lIl;
          OIl <= iOl;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Y_Out_o <= 8'd0;
        end
      else
        begin
          if (oIl > $signed(13'd235))
            Y_Out_o <= 8'd235;
          else
            if (oIl < $signed(13'd16))
              Y_Out_o <= 8'd16;
            else
              Y_Out_o <= oIl[7 : 0];
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Cb_Out_o <= 8'd0;
          Cr_Out_o <= 8'd0;
          data_Vld_out_o <= 1'b0;
        end
      else
        begin
          Cb_Out_o <= liI[7];
          Cr_Out_o <= oiI[7];
          data_Vld_out_o <= OIl;
        end
    end
  ramDualPort_sharpen #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) o1l(.clk(SYS_CLK_I),
                                                                                                                        .addr_a(Ol),
                                                                                                                        .data_a(Il),
                                                                                                                        .we_a(I0),
                                                                                                                        .addr_b(Io),
                                                                                                                        .we_b(I1),
                                                                                                                        .q_b(ill));
  ramDualPort_sharpen #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) i1l(.clk(SYS_CLK_I),
                                                                                                                        .addr_a(Ol),
                                                                                                                        .data_a(Il),
                                                                                                                        .we_a(l0),
                                                                                                                        .addr_b(lo),
                                                                                                                        .we_b(l1),
                                                                                                                        .q_b(O0l));
  ramDualPort_sharpen #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) Ool(.clk(SYS_CLK_I),
                                                                                                                        .addr_a(Ol),
                                                                                                                        .data_a(Il),
                                                                                                                        .we_a(o0),
                                                                                                                        .addr_b(oo),
                                                                                                                        .we_b(o1),
                                                                                                                        .q_b(I0l));
  ramDualPort_sharpen #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) Iol(.clk(SYS_CLK_I),
                                                                                                                        .addr_a(Ol),
                                                                                                                        .data_a(Il),
                                                                                                                        .we_a(i0),
                                                                                                                        .addr_b(io),
                                                                                                                        .we_b(i1),
                                                                                                                        .q_b(l0l));
  ramDualPort_sharpen #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH), .BUFF_DEPTH(g_DISPLAY_RESOLUTION)) lol(.clk(SYS_CLK_I),
                                                                                                                        .addr_a(Ol),
                                                                                                                        .data_a(Il),
                                                                                                                        .we_a(O1),
                                                                                                                        .addr_b(Oi),
                                                                                                                        .we_b(Oo),
                                                                                                                        .q_b(o0l));
  sharpen #(.g_DATA_BITWIDTH(g_DATAWIDTH / 3)) ool(.SYS_CLK_I(SYS_CLK_I),
                                                   .nRESET_I(RESET_n_I),
                                                   .p0_i(olI),
                                                   .p1_i(ilI),
                                                   .p2_i(O0I),
                                                   .p3_i(I0I),
                                                   .p4_i(l0I),
                                                   .p5_i(o0I),
                                                   .p6_i(i0I),
                                                   .p7_i(O1I),
                                                   .p8_i(I1I),
                                                   .DATA_OUT_o(oll));
endmodule
