/*****************************************************************************************************************************
--
--    File Name    : RGB2YCbCr.v 

--    Description  : This module implements RGB to YCbCr color conversion
--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module RGB2YCbCr #(parameter g_RGB_DATA_BIT_WIDTH = 8,
                   parameter g_YCbCr_DATA_BIT_WIDTH = 8)
                 (input wire CLOCK_I,
                  input wire RESET_N_I,
                  input wire[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Red_i,
                  input wire[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Green_i,
                  input wire[(g_RGB_DATA_BIT_WIDTH - 1) : 0] Blue_i,
                  input wire data_enable_i,
                  input wire horz_sync_i,
                  input wire vert_sync_i,
                  input wire hactive_i,
                  input wire vactive_i,
                  output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Y_out_o,
                  output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cb_out_o,
                  output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cr_out_o,
                  output reg data_enable_o,
                  output reg horz_sync_o,
                  output reg vert_sync_o,
                  output reg hactive_o,
                  output reg vactive_o);
  parameter O = 15;
  parameter I = 32768;
  parameter l = g_RGB_DATA_BIT_WIDTH + O;
  parameter o = 8421;
  parameter i = 16515;
  parameter OI = 3211;
  parameter II = 4850;
  parameter lI = 9535;
  parameter oI = 14385;
  parameter iI = 14385;
  parameter Ol = 12059;
  parameter Il = 2327;
  reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] ll;
  reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] ol;
  reg[(g_RGB_DATA_BIT_WIDTH - 1) : 0] il;
  reg[(l - 1) : 0] O0;
  reg[(l - 1) : 0] I0;
  reg[(l - 1) : 0] l0;
  reg[(l - 1) : 0] o0;
  reg[(l - 1) : 0] i0;
  reg[(l - 1) : 0] O1;
  reg[(l - 1) : 0] I1;
  reg[(l - 1) : 0] l1;
  reg[(l - 1) : 0] o1;
  reg[(l - 1) : 0] i1;
  reg[(l - 1) : 0] Oo;
  reg[(l - 1) : 0] Io;
  reg[(l - 1) : 0] lo;
  reg[(l - 1) : 0] oo;
  reg[(l - 1) : 0] io;
  reg[(l - 1) : 0] Oi;
  reg[(l - 1) : 0] Ii;
  reg[(l - 1) : 0] li;
  reg[(l - 1) : 0] oi;
  reg[(l - 1) : 0] ii;
  reg[(l - 1) : 0] OOI;
  reg[(l - 1) : 0] IOI;
  reg[(l) : 0] lOI;
  reg[(l) : 0] oOI;
  reg[(l) : 0] iOI;
  reg OII;
  reg III;
  reg lII;
  reg oII;
  reg iII;
  reg OlI;
  reg IlI;
  reg llI;
  reg olI;
  reg ilI;
  reg O0I;
  reg I0I;
  reg l0I;
  reg o0I;
  reg i0I;
  reg O1I;
  reg I1I;
  reg l1I;
  reg o1I;
  reg i1I;
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          ll <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
          ol <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
          il <= { g_RGB_DATA_BIT_WIDTH { 1'b0 } };
          OII <= 1'b0;
          iII <= 1'b0;
          olI <= 1'b0;
          l0I <= 1'b0;
          I1I <= 1'b0;
        end
      else
        begin
          ll <= Red_i;
          ol <= Green_i;
          il <= Blue_i;
          OII <= horz_sync_i;
          iII <= vert_sync_i;
          olI <= data_enable_i;
          l0I <= hactive_i;
          I1I <= vactive_i;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          O0 <= { l { 1'b0 } };
          I0 <= { l { 1'b0 } };
          l0 <= { l { 1'b0 } };
          o0 <= { l { 1'b0 } };
        end
      else
        begin
          O0 <= o * ll;
          I0 <= i * ol;
          l0 <= OI * il;
          o0 <= 16 * I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          i0 <= { l { 1'b0 } };
          O1 <= { l { 1'b0 } };
          I1 <= { l { 1'b0 } };
          Oo <= { l { 1'b0 } };
        end
      else
        begin
          i0 <= II * ll;
          O1 <= lI * ol;
          I1 <= oI * il;
          Oo <= 128 * I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          l1 <= { l { 1'b0 } };
          o1 <= { l { 1'b0 } };
          i1 <= { l { 1'b0 } };
        end
      else
        begin
          l1 <= iI * ll;
          o1 <= Ol * ol;
          i1 <= Il * il;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          III <= 1'b0;
          OlI <= 1'b0;
          ilI <= 1'b0;
          o0I <= 1'b0;
          l1I <= 1'b0;
        end
      else
        begin
          III <= OII;
          OlI <= iII;
          ilI <= olI;
          o0I <= l0I;
          l1I <= I1I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Io <= { l { 1'b0 } };
          lo <= { l { 1'b0 } };
          oo <= { l { 1'b0 } };
          io <= { l { 1'b0 } };
          Oi <= { l { 1'b0 } };
          Ii <= { l { 1'b0 } };
          li <= { l { 1'b0 } };
          IOI <= { l { 1'b0 } };
          oi <= { l { 1'b0 } };
          ii <= { l { 1'b0 } };
          OOI <= { l { 1'b0 } };
          lII <= 1'b0;
          IlI <= 1'b0;
          O0I <= 1'b0;
          i0I <= 1'b0;
          o1I <= 1'b0;
        end
      else
        begin
          Io <= O0;
          lo <= I0;
          oo <= l0;
          io <= o0;
          Oi <= i0;
          Ii <= O1;
          li <= I1;
          IOI <= Oo;
          oi <= l1;
          ii <= o1;
          OOI <= i1;
          lII <= III;
          IlI <= OlI;
          O0I <= ilI;
          i0I <= o0I;
          o1I <= l1I;
        end
    end
  always
    @(posedge CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          lOI <= { (l + 1) { 1'b0 } };
          oOI <= { (l + 1) { 1'b0 } };
          iOI <= { (l + 1) { 1'b0 } };
          oII <= 1'b0;
          llI <= 1'b0;
          I0I <= 1'b0;
          O1I <= 1'b0;
          i1I <= 1'b0;
        end
      else
        begin
          lOI <= (io + Io + lo + oo);
          oOI <= (IOI - Oi - Ii + li);
          iOI <= (IOI + oi - ii - OOI);
          oII <= lII;
          llI <= IlI;
          I0I <= O0I;
          O1I <= i0I;
          i1I <= o1I;
        end
    end
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Y_out_o <= 8'h10;
          Cb_out_o <= 8'h80;
          Cr_out_o <= 8'h80;
          horz_sync_o <= 1'b0;
          vert_sync_o <= 1'b0;
          data_enable_o <= 1'b0;
          hactive_o <= 1'b0;
          vactive_o <= 1'b0;
        end
      else
        begin
          Y_out_o <= lOI >> O;
          Cb_out_o <= oOI >> O;
          Cr_out_o <= iOI >> O;
          horz_sync_o <= oII;
          vert_sync_o <= llI;
          data_enable_o <= I0I;
          hactive_o <= O1I;
          vactive_o <= i1I;
        end
    end
endmodule
