/*****************************************************************************************************************************
--
--    File Name    : axi_arbiter.v

--    Description  : This module implements Axi Arbiter for read and write channels
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module axi_arbiter #(parameter g_AXI_AWIDTH = 32,
                     parameter g_AXI_DWIDTH = 64)
                   (input wire clock,
                    input wire nreset,
                    input wire master1_wr_req, master2_wr_req,
                    input wire split_burst_wr1, split_burst_wr2,
                    output reg write_ack1,
                               write_ack2,
                    input wire[3 : 0] tr_length_w1, tr_length_w2,
                    input wire[2 : 0] tr_size_w1, tr_size_w2,
                    input wire[1 : 0] tr_burst_w1, tr_burst_w2,
                    input wire[(g_AXI_AWIDTH - 1) : 0] tr_daddr_w1, tr_daddr_w2,
                    input wire[1 : 0] tr_type_w1, tr_type_w2,
                    input wire tr_trg_w1,
                    input wire tr_trg_w2,
                    output reg tr_done_w1,
                               tr_done_w2,
                    input wire[(g_AXI_DWIDTH - 1) : 0] wdata_din1, wdata_din2,
                    output wire buff_re1, buff_re2,
                    input wire master1_rd_req, master2_rd_req, master3_rd_req, master4_rd_req,
                    input wire split_burst_rd2, split_burst_rd3, split_burst_rd4,
                    output reg read_ack1,
                               read_ack2,
                               read_ack3,
                               read_ack4,
                    input wire[3 : 0] tr_length_r1, tr_length_r2, tr_length_r3, tr_length_r4,
                    input wire[2 : 0] tr_size_r1, tr_size_r2, tr_size_r3, tr_size_r4,
                    input wire[1 : 0] tr_burst_r1, tr_burst_r2, tr_burst_r3, tr_burst_r4,
                    input wire[(g_AXI_AWIDTH - 1) : 0] tr_saddr_r1, tr_saddr_r2, tr_saddr_r3, tr_saddr_r4,
                    input wire[1 : 0] tr_type_r1, tr_type_r2, tr_type_r3, tr_type_r4,
                    input wire tr_trg_r1,
                    input wire tr_trg_r2,
                    input wire tr_trg_r3,
                    input wire tr_trg_r4,
                    output reg tr_done_r1,
                               tr_done_r2,
                               tr_done_r3,
                               tr_done_r4,
                    output reg[(g_AXI_DWIDTH - 1) : 0] rdata_dout1,
                                                       rdata_dout2,
                                                       rdata_dout3,
                                                       rdata_dout4,
                    output reg buff_we1,
                               buff_we2,
                               buff_we3,
                               buff_we4,
                    output wire[3 : 0] tr_length_w,
                    output wire[2 : 0] tr_size_w,
                    output wire[1 : 0] tr_burst_w,
                    output wire[(g_AXI_AWIDTH - 1) : 0] tr_daddr_w,
                    output wire[1 : 0] tr_type_w,
                    output wire tr_trg_w,
                    input wire tr_done_w,
                    output reg[3 : 0] tr_length_r,
                    output reg[2 : 0] tr_size_r,
                    output reg[1 : 0] tr_burst_r,
                    output reg[(g_AXI_AWIDTH - 1) : 0] tr_saddr_r,
                    output reg[1 : 0] tr_type_r,
                    output reg tr_trg_r,
                    input wire tr_done_r,
                    input wire[(g_AXI_DWIDTH - 1) : 0] rdata_dout,
                    input wire buff_we,
                    output wire[(g_AXI_DWIDTH - 1) : 0] wdata_din,
                    input wire buff_re);
  reg[0 : 0] oiI;
  reg[1 : 0] iiI;
  reg OOl,
      IOl,
      lOl;
  parameter[0 : 0] oOl = 1'b0,
                   iOl = 1'b1;
  reg[0 : 0] OIl;
  parameter[1 : 0] IIl = 2'd0,
                   lIl = 2'd1,
                   oIl = 2'd2,
                   iIl = 2'd3;
  reg[1 : 0] Oll;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          OIl <= oOl;
          oiI <= 1'b0;
          write_ack1 <= 1'b0;
          write_ack2 <= 1'b0;
        end
      else
        begin
          case (OIl)
            oOl:
              begin
                if (master1_wr_req)
                  begin
                    oiI <= 1'b0;
                    if (tr_done_w)
                      begin
                        if (split_burst_wr1)
                          begin
                            write_ack1 <= 1'b1;
                            write_ack2 <= 1'b0;
                            OIl <= oOl;
                          end
                        else
                          begin
                            write_ack1 <= 1'b0;
                            write_ack2 <= 1'b0;
                            OIl <= iOl;
                          end
                      end
                    else
                      begin
                        write_ack1 <= 1'b1;
                        write_ack2 <= 1'b0;
                        OIl <= oOl;
                      end
                  end
                else
                  begin
                    write_ack1 <= 1'b0;
                    write_ack2 <= 1'b0;
                    OIl <= iOl;
                  end
              end
            iOl:
              begin
                if (master2_wr_req)
                  begin
                    oiI <= 1'b1;
                    if (tr_done_w)
                      begin
                        if (split_burst_wr2)
                          begin
                            write_ack1 <= 1'b0;
                            write_ack2 <= 1'b1;
                            OIl <= iOl;
                          end
                        else
                          begin
                            write_ack1 <= 1'b0;
                            write_ack2 <= 1'b0;
                            OIl <= oOl;
                          end
                      end
                    else
                      begin
                        write_ack1 <= 1'b0;
                        write_ack2 <= 1'b1;
                        OIl <= iOl;
                      end
                  end
                else
                  begin
                    write_ack1 <= 1'b0;
                    write_ack2 <= 1'b0;
                    OIl <= oOl;
                  end
              end
            default:
              OIl <= oOl;
          endcase
        end
    end
  assign tr_length_w = (oiI) ? tr_length_w2 : tr_length_w1;
  assign tr_size_w = (oiI) ? tr_size_w2 : tr_size_w1;
  assign tr_burst_w = (oiI) ? tr_burst_w2 : tr_burst_w1;
  assign tr_daddr_w = (oiI) ? tr_daddr_w2 : tr_daddr_w1;
  assign tr_type_w = (oiI) ? tr_type_w2 : tr_type_w1;
  assign tr_trg_w = (oiI) ? tr_trg_w2 : tr_trg_w1;
  assign wdata_din = (oiI) ? wdata_din2 : wdata_din1;
  assign buff_re1 = buff_re & ~oiI;
  assign buff_re2 = buff_re & oiI;
  always
    @(oiI,
      tr_done_w)
    begin
      if (oiI)
        begin
          tr_done_w2 <= tr_done_w;
          tr_done_w1 <= 1'b0;
        end
      else
        begin
          tr_done_w1 <= tr_done_w;
          tr_done_w2 <= 1'b0;
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          Oll <= IIl;
          iiI <= 2'b00;
          read_ack1 <= 1'b0;
          read_ack2 <= 1'b0;
          read_ack3 <= 1'b0;
          read_ack4 <= 1'b0;
          IOl <= 1'b0;
          OOl <= 1'b0;
          lOl <= 1'b0;
        end
      else
        begin
          case (Oll)
            IIl:
              begin
                if (master1_rd_req)
                  begin
                    iiI <= 2'b00;
                    Oll <= IIl;
                    if (tr_done_r)
                      begin
                        read_ack1 <= 1'b0;
                        read_ack2 <= 1'b0;
                        read_ack3 <= 1'b0;
                        read_ack4 <= 1'b0;
                      end
                    else
                      begin
                        read_ack1 <= 1'b1;
                        read_ack2 <= 1'b0;
                        read_ack3 <= 1'b0;
                        read_ack4 <= 1'b0;
                      end
                  end
                else
                  begin
                    read_ack1 <= 1'b0;
                    read_ack2 <= 1'b0;
                    read_ack3 <= 1'b0;
                    read_ack4 <= 1'b0;
                    if (IOl)
                      begin
                        Oll <= oIl;
                        IOl <= 1'b0;
                      end
                    else
                      if (OOl)
                        begin
                          Oll <= iIl;
                          OOl <= 1'b0;
                        end
                      else
                        if (lOl)
                          begin
                            Oll <= lIl;
                            lOl <= 1'b0;
                          end
                        else
                          begin
                            Oll <= lIl;
                          end
                  end
              end
            lIl:
              begin
                if (master2_rd_req)
                  begin
                    iiI <= 2'b01;
                    if (tr_done_r)
                      begin
                        if (split_burst_rd2)
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b1;
                            read_ack3 <= 1'b0;
                            read_ack4 <= 1'b0;
                            Oll <= lIl;
                          end
                        else
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b0;
                            read_ack3 <= 1'b0;
                            read_ack4 <= 1'b0;
                            IOl <= 1'b1;
                            Oll <= IIl;
                          end
                      end
                    else
                      begin
                        read_ack1 <= 1'b0;
                        read_ack2 <= 1'b1;
                        read_ack3 <= 1'b0;
                        read_ack4 <= 1'b0;
                        Oll <= lIl;
                      end
                  end
                else
                  begin
                    read_ack1 <= 1'b0;
                    read_ack2 <= 1'b0;
                    read_ack3 <= 1'b0;
                    read_ack4 <= 1'b0;
                    IOl <= 1'b1;
                    Oll <= IIl;
                  end
              end
            oIl:
              begin
                if (master3_rd_req)
                  begin
                    iiI <= 2'b10;
                    if (tr_done_r)
                      begin
                        if (split_burst_rd3)
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b0;
                            read_ack3 <= 1'b1;
                            read_ack4 <= 1'b0;
                            Oll <= oIl;
                          end
                        else
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b0;
                            read_ack3 <= 1'b0;
                            read_ack4 <= 1'b0;
                            OOl <= 1'b1;
                            Oll <= IIl;
                          end
                      end
                    else
                      begin
                        read_ack1 <= 1'b0;
                        read_ack2 <= 1'b0;
                        read_ack3 <= 1'b1;
                        read_ack4 <= 1'b0;
                        Oll <= oIl;
                      end
                  end
                else
                  begin
                    read_ack1 <= 1'b0;
                    read_ack2 <= 1'b0;
                    read_ack3 <= 1'b0;
                    read_ack4 <= 1'b0;
                    OOl <= 1'b1;
                    Oll <= IIl;
                  end
              end
            iIl:
              begin
                if (master4_rd_req)
                  begin
                    iiI <= 2'b11;
                    if (tr_done_r)
                      begin
                        if (split_burst_rd4)
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b0;
                            read_ack3 <= 1'b0;
                            read_ack4 <= 1'b1;
                            Oll <= iIl;
                          end
                        else
                          begin
                            read_ack1 <= 1'b0;
                            read_ack2 <= 1'b0;
                            read_ack3 <= 1'b0;
                            read_ack4 <= 1'b0;
                            lOl <= 1'b1;
                            Oll <= IIl;
                          end
                      end
                    else
                      begin
                        read_ack1 <= 1'b0;
                        read_ack2 <= 1'b0;
                        read_ack3 <= 1'b0;
                        read_ack4 <= 1'b1;
                        Oll <= iIl;
                      end
                  end
                else
                  begin
                    read_ack1 <= 1'b0;
                    read_ack2 <= 1'b0;
                    read_ack3 <= 1'b0;
                    read_ack4 <= 1'b0;
                    lOl <= 1'b1;
                    Oll <= IIl;
                  end
              end
            default:
              Oll <= IIl;
          endcase
        end
    end
  always
    @(iiI,
      tr_length_r1,
      tr_length_r2,
      tr_length_r3,
      tr_length_r4)
    begin
      if (iiI == 2'b00)
        tr_length_r <= tr_length_r1;
      else
        if (iiI == 2'b01)
          tr_length_r <= tr_length_r2;
        else
          if (iiI == 2'b10)
            tr_length_r <= tr_length_r3;
          else
            if (iiI == 2'b11)
              tr_length_r <= tr_length_r4;
            else
              tr_length_r <= tr_length_r1;
    end
  always
    @(iiI,
      tr_size_r1,
      tr_size_r2,
      tr_size_r3,
      tr_size_r4)
    begin
      if (iiI == 2'b00)
        tr_size_r <= tr_size_r1;
      else
        if (iiI == 2'b01)
          tr_size_r <= tr_size_r2;
        else
          if (iiI == 2'b10)
            tr_size_r <= tr_size_r3;
          else
            if (iiI == 2'b11)
              tr_size_r <= tr_size_r4;
            else
              tr_size_r <= tr_size_r1;
    end
  always
    @(iiI,
      tr_burst_r1,
      tr_burst_r2,
      tr_burst_r3,
      tr_burst_r4)
    begin
      if (iiI == 2'b00)
        tr_burst_r <= tr_burst_r1;
      else
        if (iiI == 2'b01)
          tr_burst_r <= tr_burst_r2;
        else
          if (iiI == 2'b10)
            tr_burst_r <= tr_burst_r3;
          else
            if (iiI == 2'b11)
              tr_burst_r <= tr_burst_r4;
            else
              tr_burst_r <= tr_burst_r1;
    end
  always
    @(iiI,
      tr_saddr_r1,
      tr_saddr_r2,
      tr_saddr_r3,
      tr_saddr_r4)
    begin
      if (iiI == 2'b00)
        tr_saddr_r <= tr_saddr_r1;
      else
        if (iiI == 2'b01)
          tr_saddr_r <= tr_saddr_r2;
        else
          if (iiI == 2'b10)
            tr_saddr_r <= tr_saddr_r3;
          else
            if (iiI == 2'b11)
              tr_saddr_r <= tr_saddr_r4;
            else
              tr_saddr_r <= tr_saddr_r1;
    end
  always
    @(iiI,
      tr_type_r1,
      tr_type_r2,
      tr_type_r3,
      tr_type_r4)
    begin
      if (iiI == 2'b00)
        tr_type_r <= tr_type_r1;
      else
        if (iiI == 2'b01)
          tr_type_r <= tr_type_r2;
        else
          if (iiI == 2'b10)
            tr_type_r <= tr_type_r3;
          else
            if (iiI == 2'b11)
              tr_type_r <= tr_type_r4;
            else
              tr_type_r <= tr_type_r1;
    end
  always
    @(iiI,
      tr_trg_r1,
      tr_trg_r2,
      tr_trg_r3,
      tr_trg_r4)
    begin
      if (iiI == 2'b00)
        tr_trg_r <= tr_trg_r1;
      else
        if (iiI == 2'b01)
          tr_trg_r <= tr_trg_r2;
        else
          if (iiI == 2'b10)
            tr_trg_r <= tr_trg_r3;
          else
            if (iiI == 2'b11)
              tr_trg_r <= tr_trg_r4;
            else
              tr_trg_r <= tr_trg_r1;
    end
  always
    @(iiI,
      tr_done_r)
    begin
      if (iiI == 2'b00)
        begin
          tr_done_r1 <= tr_done_r;
          tr_done_r2 <= 1'b0;
          tr_done_r3 <= 1'b0;
          tr_done_r4 <= 1'b0;
        end
      else
        if (iiI == 2'b01)
          begin
            tr_done_r1 <= 1'b0;
            tr_done_r2 <= tr_done_r;
            tr_done_r3 <= 1'b0;
            tr_done_r4 <= 1'b0;
          end
        else
          if (iiI == 2'b10)
            begin
              tr_done_r2 <= 1'b0;
              tr_done_r1 <= 1'b0;
              tr_done_r3 <= tr_done_r;
              tr_done_r4 <= 1'b0;
            end
          else
            if (iiI == 2'b11)
              begin
                tr_done_r2 <= 1'b0;
                tr_done_r1 <= 1'b0;
                tr_done_r3 <= 1'b0;
                tr_done_r4 <= tr_done_r;
              end
            else
              begin
                tr_done_r1 <= tr_done_r;
                tr_done_r2 <= 1'b0;
                tr_done_r3 <= 1'b0;
                tr_done_r4 <= 1'b0;
              end
    end
  always
    @(iiI,
      buff_we)
    begin
      if (iiI == 2'b00)
        begin
          buff_we1 <= buff_we;
          buff_we2 <= 1'b0;
          buff_we3 <= 1'b0;
          buff_we4 <= 1'b0;
        end
      else
        if (iiI == 2'b01)
          begin
            buff_we1 <= 1'b0;
            buff_we2 <= buff_we;
            buff_we3 <= 1'b0;
            buff_we4 <= 1'b0;
          end
        else
          if (iiI == 2'b10)
            begin
              buff_we1 <= 1'b0;
              buff_we2 <= 1'b0;
              buff_we3 <= buff_we;
              buff_we4 <= 1'b0;
            end
          else
            if (iiI == 2'b11)
              begin
                buff_we1 <= 1'b0;
                buff_we2 <= 1'b0;
                buff_we3 <= 1'b0;
                buff_we4 <= buff_we;
              end
            else
              begin
                buff_we1 <= buff_we;
                buff_we2 <= 1'b0;
                buff_we3 <= 1'b0;
                buff_we4 <= 1'b0;
              end
    end
  always
    @(iiI,
      rdata_dout)
    begin
      if (iiI == 2'b00)
        begin
          rdata_dout1 <= rdata_dout;
          rdata_dout2 <= { g_AXI_DWIDTH { 1'b0 } };
          rdata_dout3 <= { g_AXI_DWIDTH { 1'b0 } };
          rdata_dout4 <= { g_AXI_DWIDTH { 1'b0 } };
        end
      else
        if (iiI == 2'b01)
          begin
            rdata_dout1 <= { g_AXI_DWIDTH { 1'b0 } };
            rdata_dout2 <= rdata_dout;
            rdata_dout3 <= { g_AXI_DWIDTH { 1'b0 } };
            rdata_dout4 <= { g_AXI_DWIDTH { 1'b0 } };
          end
        else
          if (iiI == 2'b10)
            begin
              rdata_dout1 <= { g_AXI_DWIDTH { 1'b0 } };
              rdata_dout2 <= { g_AXI_DWIDTH { 1'b0 } };
              rdata_dout3 <= rdata_dout;
              rdata_dout4 <= { g_AXI_DWIDTH { 1'b0 } };
            end
          else
            if (iiI == 2'b11)
              begin
                rdata_dout1 <= { g_AXI_DWIDTH { 1'b0 } };
                rdata_dout2 <= { g_AXI_DWIDTH { 1'b0 } };
                rdata_dout3 <= { g_AXI_DWIDTH { 1'b0 } };
                rdata_dout4 <= rdata_dout;
              end
            else
              begin
                rdata_dout1 <= rdata_dout;
                rdata_dout2 <= { g_AXI_DWIDTH { 1'b0 } };
                rdata_dout3 <= { g_AXI_DWIDTH { 1'b0 } };
                rdata_dout4 <= { g_AXI_DWIDTH { 1'b0 } };
              end
    end
endmodule
