/*****************************************************************************************************************************
--
--    File Name    : data_packer_32_64.v

--    Description  : This module implements 32-bit to 64-bit packer for write channels
--                   
-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team


-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module pack_32_64 #(parameter g_AXI_DWIDTH = 64,
                    parameter g_AXI_BUFF_AWIDTH = 10,
                    parameter g_WR_CHANNEL_VIDEO_DATA_WIDTH = 32)
                  (input wire clock,
                   input wire nreset,
                   input wire start_pack,
                   input wire[(g_WR_CHANNEL_VIDEO_DATA_WIDTH - 1) : 0] video_data_in,
                   input wire data_valid,
                   output wire buff_we,
                   output reg[(g_AXI_BUFF_AWIDTH - 1) : 0] buff_waddr,
                   output wire[(g_AXI_DWIDTH - 1) : 0] buff_data_o);
  reg[(g_AXI_DWIDTH - 1) : 0] oll;
  reg O0l;
  parameter[0 : 0] l0l = 1'b0,
                   o0l = 1'd1;
  reg[0 : 0] Iol;
  reg I0l;
  assign buff_data_o = oll;
  assign buff_we = O0l;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        Iol <= l0l;
      else
        if (start_pack)
          begin
            if (data_valid)
              begin
                case (Iol)
                  l0l:
                    begin
                      Iol <= o0l;
                    end
                  o0l:
                    begin
                      Iol <= l0l;
                    end
                  default:
                    Iol <= l0l;
                endcase
              end
            else
              begin
                Iol <= Iol;
              end
          end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          oll <= { g_AXI_DWIDTH { 1'b0 } };
          O0l <= 1'b0;
          I0l <= 1'b1;
          buff_waddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
        end
      else
        if (start_pack)
          begin
            if (data_valid)
              begin
                case (Iol)
                  l0l:
                    begin
                      oll <= { video_data_in, oll[63 : 32] };
                      O0l <= 1'b0;
                    end
                  o0l:
                    begin
                      oll <= { video_data_in, oll[63 : 32] };
                      O0l <= 1'b1;
                      if (I0l)
                        begin
                          I0l <= 1'b0;
                          buff_waddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                        end
                      else
                        begin
                          I0l <= 1'b0;
                          buff_waddr <= buff_waddr + 1'b1;
                        end
                    end
                  default:
                    begin
                    end
                endcase
              end
            else
              begin
                oll <= oll;
                O0l <= 1'b0;
                I0l <= I0l;
              end
          end
        else
          begin
            oll <= { g_AXI_DWIDTH { 1'b0 } };
            O0l <= 1'b0;
            I0l <= 1'b1;
          end
    end
endmodule
