/*****************************************************************************************************************************
--
--    File Name    : read_channel3_top.v  

--    Description  : This module is the read channel 3 top-level
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module read_channel3_top #(parameter g_AXI_AWIDTH = 32,
                           parameter g_AXI_DWIDTH = 64,
                           parameter g_AXI_BUFF_AWIDTH = 12,
                           parameter g_HORIZONTAL_RESOLUTION = 1280,
                           parameter g_RD_CHANNEL3_VIDEO_DATA_WIDTH = 32,
                           parameter g_RD_CHANNEL3_BUFFER_LINE_STORAGE = 1)
                         (input wire clock,
                          input wire nreset,
                          input wire arbitr_ack,
                          input wire[(g_AXI_DWIDTH - 1) : 0] axi_data_in,
                          input wire axi_if_wen,
                          input wire axi_tr_done,
                          input wire[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_3,
                          input wire rd_req_3,
                          input wire[(g_AXI_AWIDTH - 1) : 0] start_read_addr_3,
                          output wire[3 : 0] axi_burst_len,
                          output wire[1 : 0] axi_burst_typ,
                          output wire[(g_AXI_AWIDTH - 1) : 0] axi_saddr,
                          output wire axi_start,
                          output wire[1 : 0] axi_tr_dir,
                          output wire[2 : 0] axi_tr_size,
                          output wire master_rd_req,
                          output wire rd_ack_3,
                          output wire rd_burst_split,
                          output wire rd_done_3,
                          output wire rdata_valid_3,
                          output wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] video_rdata_3);
  parameter iI1I = ((g_HORIZONTAL_RESOLUTION * g_RD_CHANNEL3_VIDEO_DATA_WIDTH * g_RD_CHANNEL3_BUFFER_LINE_STORAGE) / g_AXI_DWIDTH);
  wire[(g_AXI_DWIDTH - 1) : 0] oO0I;
  wire[3 : 0] iO0I;
  wire[1 : 0] OI0I;
  wire[(g_AXI_BUFF_AWIDTH - 1) : 0] II0I;
  wire[(g_AXI_DWIDTH - 1) : 0] lI0I;
  wire oI0I;
  wire iI0I;
  wire[(g_AXI_AWIDTH - 1) : 0] Ol0I;
  wire Il0I;
  wire[1 : 0] ll0I;
  wire[2 : 0] ol0I;
  wire[4 : 0] il0I;
  wire[(g_AXI_AWIDTH - 1) : 0] O00I;
  wire[(g_AXI_BUFF_AWIDTH - 1) : 0] I00I;
  wire O10I;
  wire I10I;
  wire l10I;
  wire l0i;
  wire o10I;
  wire I1i;
  wire ioi;
  wire[(g_AXI_BUFF_AWIDTH - 1) : 0] Ol1I;
  wire Il1I;
  wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] I0lI;
  wire i1II;
  wire l1II;
  wire Io0I;
  wire lo0I;
  wire[1 : 0] oo0I;
  wire[3 : 0] io0I;
  wire[1 : 0] Oi0I;
  wire[(g_AXI_AWIDTH - 1) : 0] Ii0I;
  wire li0I;
  wire[2 : 0] oi0I;
  wire O1II;
  wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] O1lI;
  assign i1II = l0i;
  assign rd_ack_3 = i1II;
  assign l1II = I1i;
  assign rd_done_3 = l1II;
  assign Io0I = Il0I;
  assign axi_start = Io0I;
  assign lo0I = l10I;
  assign master_rd_req = lo0I;
  assign oo0I = OI0I;
  assign axi_burst_typ = oo0I;
  assign io0I = iO0I;
  assign axi_burst_len = io0I;
  assign Oi0I = ll0I;
  assign axi_tr_dir = Oi0I;
  assign Ii0I = Ol0I;
  assign axi_saddr = Ii0I;
  assign li0I = o10I;
  assign rd_burst_split = li0I;
  assign oi0I = ol0I;
  assign axi_tr_size = oi0I;
  assign O1II = ioi;
  assign rdata_valid_3 = O1II;
  assign O1lI = I0lI;
  assign video_rdata_3 = O1lI;
  axi_buffer #(.g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH), .AXI_BUFF_DEPTH(iI1I), .g_AXI_DWIDTH(g_AXI_DWIDTH)) ii0I(.wr_data_i(lI0I),
                                                                                                               .rd_addr(Ol1I),
                                                                                                               .wr_addr(II0I),
                                                                                                               .we(oI0I),
                                                                                                               .wclock(clock),
                                                                                                               .rclock(clock),
                                                                                                               .rd_data_o(oO0I));
  AXI_master_read #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) iO1I(.clock(clock),
                                                                                                                          .nreset(nreset),
                                                                                                                          .axi_rd_ddr_addr(O00I),
                                                                                                                          .burst_length(il0I),
                                                                                                                          .start_axi_tr(O10I),
                                                                                                                          .axi_tr_done(axi_tr_done),
                                                                                                                          .arbitr_ack(arbitr_ack),
                                                                                                                          .axi_data_in(axi_data_in),
                                                                                                                          .axi_if_wen(axi_if_wen),
                                                                                                                          .locs_to_read(I00I),
                                                                                                                          .transf_compl(iI0I),
                                                                                                                          .axi_saddr(Ol0I),
                                                                                                                          .axi_tr_size(ol0I),
                                                                                                                          .axi_burst_typ(OI0I),
                                                                                                                          .axi_burst_len(iO0I),
                                                                                                                          .axi_tr_dir(ll0I),
                                                                                                                          .axi_start(Il0I),
                                                                                                                          .master_rd_req(l10I),
                                                                                                                          .axi_buff_data_o(lI0I),
                                                                                                                          .axi_buff_addr(II0I),
                                                                                                                          .axi_buff_wen(oI0I));
  ddr_read_contrl #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) OI1I(.clock(clock),
                                                                                             .nreset(nreset),
                                                                                             .rd_req_i(rd_req_3),
                                                                                             .ddr_read_start_address_i(start_read_addr_3),
                                                                                             .bytes_to_read_i(bytes_to_read_3),
                                                                                             .unpacker_compl(Il1I),
                                                                                             .axi_tr_done(iI0I),
                                                                                             .rd_ack_o(l0i),
                                                                                             .rd_done_o(I1i),
                                                                                             .start_unpack(I10I),
                                                                                             .locs_to_read(I00I),
                                                                                             .axi_rd_ddr_addr(O00I),
                                                                                             .axi_burst_len(il0I),
                                                                                             .start_axi_tr(O10I),
                                                                                             .rd_burst_split(o10I));
  generate
    if (g_RD_CHANNEL3_VIDEO_DATA_WIDTH == 32)
      begin
        unpack_64_32 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_RD_CHANNEL_VIDEO_DATA_WIDTH(g_RD_CHANNEL3_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) oI1I(.clock(clock),
                                                                                                                                                                .nreset(nreset),
                                                                                                                                                                .start_unpack(I10I),
                                                                                                                                                                .locs_to_read(I00I),
                                                                                                                                                                .buff_data_in(oO0I),
                                                                                                                                                                .buff_raddr(Ol1I),
                                                                                                                                                                .video_data(I0lI),
                                                                                                                                                                .data_valid(ioi),
                                                                                                                                                                .unpacker_compl(Il1I));
      end
    else
      if (g_RD_CHANNEL3_VIDEO_DATA_WIDTH == 8)
        begin
          unpack_64_8 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_RD_CHANNEL_VIDEO_DATA_WIDTH(g_RD_CHANNEL3_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) lI1I(.clock(clock),
                                                                                                                                                                 .nreset(nreset),
                                                                                                                                                                 .start_unpack(I10I),
                                                                                                                                                                 .locs_to_read(I00I),
                                                                                                                                                                 .buff_data_in(oO0I),
                                                                                                                                                                 .buff_raddr(Ol1I),
                                                                                                                                                                 .video_data(I0lI),
                                                                                                                                                                 .data_valid(ioi),
                                                                                                                                                                 .unpacker_compl(Il1I));
        end
      else
        if (g_RD_CHANNEL3_VIDEO_DATA_WIDTH == 24)
          begin
            unpack_64_24 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_RD_CHANNEL_VIDEO_DATA_WIDTH(g_RD_CHANNEL3_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) II1I(.clock(clock),
                                                                                                                                                                    .nreset(nreset),
                                                                                                                                                                    .start_unpack(I10I),
                                                                                                                                                                    .locs_to_read(I00I),
                                                                                                                                                                    .buff_data_in(oO0I),
                                                                                                                                                                    .buff_raddr(Ol1I),
                                                                                                                                                                    .video_data(I0lI),
                                                                                                                                                                    .data_valid(ioi),
                                                                                                                                                                    .unpacker_compl(Il1I));
          end
  endgenerate
endmodule
