/*****************************************************************************************************************************
--
--    File Name    : embsync_detect.v 

--    Description  : This module implements detection of MIPI Sync code.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
module embsync_detect #(parameter g_LANE_WIDTH = 4)
                      (input oOl,
                       input CLOCK_I,
                       input HS,
                       input[7 : 0] L0_DATA_I,
                       input[7 : 0] L1_DATA_I,
                       input[7 : 0] L2_DATA_I,
                       input[7 : 0] L3_DATA_I,
                       input L0_LP_DATA_I,
                       input L0_LP_DATA_N_I,
                       input L1_LP_DATA_I,
                       input L1_LP_DATA_N_I,
                       input L2_LP_DATA_I,
                       input L2_LP_DATA_N_I,
                       input L3_LP_DATA_I,
                       input L3_LP_DATA_N_I,
                       output[7 : 0] L0_DATA_O,
                       output[7 : 0] L1_DATA_O,
                       output[7 : 0] L2_DATA_O,
                       output[7 : 0] L3_DATA_O);
  reg[7 : 0] iOl,
             OIl,
             IIl,
             lIl;
  reg[7 : 0] oIl,
             iIl,
             Oll,
             Ill;
  reg[7 : 0] lll,
             oll,
             ill,
             O0l;
  reg[7 : 0] I0l,
             l0l,
             o0l,
             i0l;
  reg[7 : 0] O1l,
             I1l,
             l1l,
             o1l;
  reg[7 : 0] i1l,
             Ool,
             Iol,
             lol;
  reg[7 : 0] ool,
             iol,
             Oil,
             Iil;
  reg[2 : 0] lil,
             oil,
             iil,
             OO0;
  reg[7 : 0] IO0,
             lO0,
             oO0,
             iO0;
  wire[7 : 0] OI0,
              II0,
              lI0,
              oI0;
  reg iI0,
      Ol0,
      Il0,
      ll0;
  reg ol0,
      il0,
      O00,
      I00;
  reg[7 : 0] l00,
             o00,
             i00,
             O10,
             I10,
             l10,
             o10,
             i10;
  reg[7 : 0] Oo0,
             Io0,
             lo0,
             oo0,
             io0,
             Oi0,
             Ii0,
             li0;
  reg[7 : 0] oi0,
             ii0,
             OO1,
             IO1,
             lO1,
             oO1,
             iO1,
             OI1;
  reg[7 : 0] II1,
             lI1,
             oI1,
             iI1,
             Ol1,
             Il1,
             ll1,
             ol1;
  reg[7 : 0] il1,
             O01,
             I01,
             l01,
             o01,
             i01,
             O11,
             I11;
  reg[7 : 0] l11,
             o11,
             i11,
             Oo1,
             Io1,
             lo1,
             oo1,
             io1;
  reg[7 : 0] Oi1,
             Ii1,
             li1,
             oi1,
             ii1,
             OOo,
             IOo,
             lOo;
  reg[7 : 0] oOo,
             iOo,
             OIo,
             IIo,
             lIo,
             oIo,
             iIo,
             Olo;
  reg[7 : 0] Ilo,
             llo,
             olo,
             ilo,
             O0o,
             I0o,
             l0o,
             o0o,
             i0o,
             O1o,
             I1o,
             l1o,
             o1o,
             i1o,
             Ooo,
             Ioo;
  reg loo,
      ooo,
      ioo,
      Oio,
      Iio,
      lio,
      oio,
      iio,
      OOi;
  reg IOi,
      lOi,
      oOi,
      iOi,
      OIi;
  reg IIi;
  reg[10 : 0] lIi;
  parameter oIi = 8'b00011101;
  wire iIi;
  wire Oli;
  wire Ili;
  wire lli;
  reg[3 : 0] oli;
  reg[3 : 0] ili;
  reg[3 : 0] O0i;
  reg[3 : 0] I0i;
  assign iIi = ~(oli[3] & oli[2] & oli[1] & oli[0] & L0_LP_DATA_N_I);
  assign Oli = ~(ili[3] & ili[2] & ili[1] & ili[0] & L1_LP_DATA_N_I);
  assign Ili = ~(O0i[3] & O0i[2] & O0i[1] & O0i[0] & L2_LP_DATA_N_I);
  assign lli = ~(I0i[3] & I0i[2] & I0i[1] & I0i[0] & L3_LP_DATA_N_I);
  always
    @(posedge CLOCK_I or
      negedge oOl)
    if (!oOl)
      begin
        oli <= 4'hF;
        ili <= 4'hF;
        O0i <= 4'hF;
        I0i <= 4'hF;
      end
    else
      begin
        oli <= { oli[2 : 0], L0_LP_DATA_N_I };
        ili <= { ili[2 : 0], L1_LP_DATA_N_I };
        O0i <= { O0i[2 : 0], L2_LP_DATA_N_I };
        I0i <= { I0i[2 : 0], L3_LP_DATA_N_I };
      end
  always
    @(posedge CLOCK_I or
      negedge oOl)
    if (!oOl)
      begin
        IOi <= 0;
        lOi <= 0;
        oOi <= 0;
        iOi <= 0;
        OIi <= 0;
      end
    else
      begin
        IOi <= HS;
        lOi <= IOi;
        oOi <= lOi;
        iOi <= oOi;
        OIi <= iOi;
      end
  generate
    if (g_LANE_WIDTH > 0)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                iOl <= 0;
                oIl <= 0;
                lll <= 0;
                I0l <= 0;
                O1l <= 0;
                i1l <= 0;
                ool <= 0;
              end
            else
              if (!iIi)
                begin
                  iOl <= 0;
                  oIl <= 0;
                  lll <= 0;
                  I0l <= 0;
                  O1l <= 0;
                  i1l <= 0;
                  ool <= 0;
                end
              else
                begin
                  iOl <= { L0_DATA_I[0], L0_DATA_I[1], L0_DATA_I[2], L0_DATA_I[3], L0_DATA_I[4], L0_DATA_I[5], L0_DATA_I[6], L0_DATA_I[7] };
                  oIl <= iOl[7 : 0];
                  lll <= oIl;
                  I0l <= lll;
                  O1l <= I0l;
                  i1l <= O1l;
                  ool <= i1l;
                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                lil <= 0;
                iI0 <= 0;
              end
            else
              if (!iIi)
                begin
                  lil <= 0;
                  iI0 <= 0;
                end
              else
                if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000000) && (iOl[7 : 0] == 8'b00011101))
                  begin
                    lil <= 0;
                    iI0 <= 1;
                  end
                else
                  if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000000) && (iOl[7 : 1] == 7'b0011101))
                    begin
                      lil <= 1;
                      iI0 <= 1;
                    end
                  else
                    if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000000) && (iOl[7 : 2] == 6'b011101))
                      begin
                        lil <= 2;
                        iI0 <= 1;
                      end
                    else
                      if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000000) && (iOl[7 : 3] == 5'b11101))
                        begin
                          lil <= 3;
                          iI0 <= 1;
                        end
                      else
                        if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000001) && (iOl[7 : 4] == 4'b1101))
                          begin
                            lil <= 4;
                            iI0 <= 1;
                          end
                        else
                          if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000011) && (iOl[7 : 5] == 3'b101))
                            begin
                              lil <= 5;
                              iI0 <= 1;
                            end
                          else
                            if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00000111) && (iOl[7 : 6] == 2'b01))
                              begin
                                lil <= 6;
                                iI0 <= 1;
                              end
                            else
                              if ((O1l == 8'h00) && (I0l == 8'h00) && (lll == 8'h00) && (oIl == 8'b00001110) && (iOl[7 : 7] == 1'b1))
                                begin
                                  lil <= 7;
                                  iI0 <= 1;
                                end
                              else
                                begin
                                  lil <= lil;
                                  iI0 <= iI0;
                                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              IO0 <= 0;
            else
              if (!iIi)
                IO0 <= 0;
              else
                if (iI0)
                  case (lil)
                    0:
                      IO0 <= ool;
                    1:
                      IO0 <= { ool[0], i1l[7 : 1] };
                    2:
                      IO0 <= { ool[1 : 0], i1l[7 : 2] };
                    3:
                      IO0 <= { ool[2 : 0], i1l[7 : 3] };
                    4:
                      IO0 <= { ool[3 : 0], i1l[7 : 4] };
                    5:
                      IO0 <= { ool[4 : 0], i1l[7 : 5] };
                    6:
                      IO0 <= { ool[5 : 0], i1l[7 : 6] };
                    7:
                      IO0 <= { ool[6 : 0], i1l[7 : 7] };
                    default:
                      IO0 <= 0;
                  endcase
                else
                  IO0 <= 0;
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              ol0 <= 0;
            else
              if (!iIi)
                ol0 <= 0;
              else
                ol0 <= iI0;
          end
        assign OI0 = { IO0[0], IO0[1], IO0[2], IO0[3], IO0[4], IO0[5], IO0[6], IO0[7] };
      end
  endgenerate
  generate
    if (g_LANE_WIDTH > 1)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                OIl <= 0;
                iIl <= 0;
                oll <= 0;
                l0l <= 0;
                I1l <= 0;
                Ool <= 0;
                iol <= 0;
              end
            else
              if (!Oli)
                begin
                  OIl <= 0;
                  iIl <= 0;
                  oll <= 0;
                  l0l <= 0;
                  I1l <= 0;
                  Ool <= 0;
                  iol <= 0;
                end
              else
                begin
                  OIl <= { L1_DATA_I[0], L1_DATA_I[1], L1_DATA_I[2], L1_DATA_I[3], L1_DATA_I[4], L1_DATA_I[5], L1_DATA_I[6], L1_DATA_I[7] };
                  iIl <= OIl[7 : 0];
                  oll <= iIl;
                  l0l <= oll;
                  I1l <= l0l;
                  Ool <= I1l;
                  iol <= Ool;
                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                oil <= 0;
                Ol0 <= 0;
              end
            else
              if (!Oli)
                begin
                  oil <= 0;
                  Ol0 <= 0;
                end
              else
                if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000000) && (OIl[7 : 0] == 8'b00011101))
                  begin
                    oil <= 0;
                    Ol0 <= 1;
                  end
                else
                  if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000000) && (OIl[7 : 1] == 7'b0011101))
                    begin
                      oil <= 1;
                      Ol0 <= 1;
                    end
                  else
                    if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000000) && (OIl[7 : 2] == 6'b011101))
                      begin
                        oil <= 2;
                        Ol0 <= 1;
                      end
                    else
                      if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000000) && (OIl[7 : 3] == 5'b11101))
                        begin
                          oil <= 3;
                          Ol0 <= 1;
                        end
                      else
                        if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000001) && (OIl[7 : 4] == 4'b1101))
                          begin
                            oil <= 4;
                            Ol0 <= 1;
                          end
                        else
                          if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000011) && (OIl[7 : 5] == 3'b101))
                            begin
                              oil <= 5;
                              Ol0 <= 1;
                            end
                          else
                            if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00000111) && (OIl[7 : 6] == 2'b01))
                              begin
                                oil <= 6;
                                Ol0 <= 1;
                              end
                            else
                              if ((I1l == 8'h00) && (l0l == 8'h00) && (oll == 8'h00) && (iIl == 8'b00001110) && (OIl[7 : 7] == 1'b1))
                                begin
                                  oil <= 7;
                                  Ol0 <= 1;
                                end
                              else
                                begin
                                  oil <= oil;
                                  Ol0 <= Ol0;
                                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              lO0 <= 0;
            else
              if (!Oli)
                lO0 <= 0;
              else
                if (Ol0)
                  case (oil)
                    0:
                      lO0 <= iol;
                    1:
                      lO0 <= { iol[0], Ool[7 : 1] };
                    2:
                      lO0 <= { iol[1 : 0], Ool[7 : 2] };
                    3:
                      lO0 <= { iol[2 : 0], Ool[7 : 3] };
                    4:
                      lO0 <= { iol[3 : 0], Ool[7 : 4] };
                    5:
                      lO0 <= { iol[4 : 0], Ool[7 : 5] };
                    6:
                      lO0 <= { iol[5 : 0], Ool[7 : 6] };
                    7:
                      lO0 <= { iol[6 : 0], Ool[7 : 7] };
                    default:
                      lO0 <= 0;
                  endcase
                else
                  lO0 <= 0;
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              il0 <= 0;
            else
              if (!Oli)
                il0 <= 0;
              else
                il0 <= Ol0;
          end
        assign II0 = { lO0[0], lO0[1], lO0[2], lO0[3], lO0[4], lO0[5], lO0[6], lO0[7] };
      end
  endgenerate
  generate
    if (g_LANE_WIDTH > 2)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                IIl <= 0;
                Oll <= 0;
                ill <= 0;
                o0l <= 0;
                l1l <= 0;
                Iol <= 0;
                Oil <= 0;
              end
            else
              if (!Ili)
                begin
                  IIl <= 0;
                  Oll <= 0;
                  ill <= 0;
                  o0l <= 0;
                  l1l <= 0;
                  Iol <= 0;
                  Oil <= 0;
                end
              else
                begin
                  IIl <= { L2_DATA_I[0], L2_DATA_I[1], L2_DATA_I[2], L2_DATA_I[3], L2_DATA_I[4], L2_DATA_I[5], L2_DATA_I[6], L2_DATA_I[7] };
                  Oll <= IIl[7 : 0];
                  ill <= Oll;
                  o0l <= ill;
                  l1l <= o0l;
                  Iol <= l1l;
                  Oil <= Iol;
                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                iil <= 0;
                Il0 <= 0;
              end
            else
              if (!Ili)
                begin
                  iil <= 0;
                  Il0 <= 0;
                end
              else
                if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000000) && (IIl[7 : 0] == 8'b00011101))
                  begin
                    iil <= 0;
                    Il0 <= 1;
                  end
                else
                  if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000000) && (IIl[7 : 1] == 7'b0011101))
                    begin
                      iil <= 1;
                      Il0 <= 1;
                    end
                  else
                    if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000000) && (IIl[7 : 2] == 6'b011101))
                      begin
                        iil <= 2;
                        Il0 <= 1;
                      end
                    else
                      if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000000) && (IIl[7 : 3] == 5'b11101))
                        begin
                          iil <= 3;
                          Il0 <= 1;
                        end
                      else
                        if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000001) && (IIl[7 : 4] == 4'b1101))
                          begin
                            iil <= 4;
                            Il0 <= 1;
                          end
                        else
                          if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000011) && (IIl[7 : 5] == 3'b101))
                            begin
                              iil <= 5;
                              Il0 <= 1;
                            end
                          else
                            if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00000111) && (IIl[7 : 6] == 2'b01))
                              begin
                                iil <= 6;
                                Il0 <= 1;
                              end
                            else
                              if ((l1l == 8'h00) && (o0l == 8'h00) && (ill == 8'h00) && (Oll == 8'b00001110) && (IIl[7 : 7] == 1'b1))
                                begin
                                  iil <= 7;
                                  Il0 <= 1;
                                end
                              else
                                begin
                                  iil <= iil;
                                  Il0 <= Il0;
                                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              oO0 <= 0;
            else
              if (!Ili)
                oO0 <= 0;
              else
                if (Il0)
                  case (iil)
                    0:
                      oO0 <= Oil;
                    1:
                      oO0 <= { Oil[0], Iol[7 : 1] };
                    2:
                      oO0 <= { Oil[1 : 0], Iol[7 : 2] };
                    3:
                      oO0 <= { Oil[2 : 0], Iol[7 : 3] };
                    4:
                      oO0 <= { Oil[3 : 0], Iol[7 : 4] };
                    5:
                      oO0 <= { Oil[4 : 0], Iol[7 : 5] };
                    6:
                      oO0 <= { Oil[5 : 0], Iol[7 : 6] };
                    7:
                      oO0 <= { Oil[6 : 0], Iol[7 : 7] };
                    default:
                      oO0 <= 0;
                  endcase
                else
                  oO0 <= 0;
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              O00 <= 0;
            else
              if (!Ili)
                O00 <= 0;
              else
                O00 <= Il0;
          end
        assign lI0 = { oO0[0], oO0[1], oO0[2], oO0[3], oO0[4], oO0[5], oO0[6], oO0[7] };
      end
  endgenerate
  generate
    if (g_LANE_WIDTH > 3)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                lIl <= 0;
                Ill <= 0;
                O0l <= 0;
                i0l <= 0;
                o1l <= 0;
                lol <= 0;
                Iil <= 0;
              end
            else
              if (!lli)
                begin
                  lIl <= 0;
                  Ill <= 0;
                  O0l <= 0;
                  i0l <= 0;
                  o1l <= 0;
                  lol <= 0;
                  Iil <= 0;
                end
              else
                begin
                  lIl <= { L3_DATA_I[0], L3_DATA_I[1], L3_DATA_I[2], L3_DATA_I[3], L3_DATA_I[4], L3_DATA_I[5], L3_DATA_I[6], L3_DATA_I[7] };
                  Ill <= lIl[7 : 0];
                  O0l <= Ill;
                  i0l <= O0l;
                  o1l <= i0l;
                  lol <= o1l;
                  Iil <= lol;
                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                OO0 <= 0;
                ll0 <= 0;
              end
            else
              if (!lli)
                begin
                  OO0 <= 0;
                  ll0 <= 0;
                end
              else
                if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000000) && (lIl[7 : 0] == 8'b00011101))
                  begin
                    OO0 <= 0;
                    ll0 <= 1;
                  end
                else
                  if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000000) && (lIl[7 : 1] == 7'b0011101))
                    begin
                      OO0 <= 1;
                      ll0 <= 1;
                    end
                  else
                    if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000000) && (lIl[7 : 2] == 6'b011101))
                      begin
                        OO0 <= 2;
                        ll0 <= 1;
                      end
                    else
                      if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000000) && (lIl[7 : 3] == 5'b11101))
                        begin
                          OO0 <= 3;
                          ll0 <= 1;
                        end
                      else
                        if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000001) && (lIl[7 : 4] == 4'b1101))
                          begin
                            OO0 <= 4;
                            ll0 <= 1;
                          end
                        else
                          if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000011) && (lIl[7 : 5] == 3'b101))
                            begin
                              OO0 <= 5;
                              ll0 <= 1;
                            end
                          else
                            if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00000111) && (lIl[7 : 6] == 2'b01))
                              begin
                                OO0 <= 6;
                                ll0 <= 1;
                              end
                            else
                              if ((o1l == 8'h00) && (i0l == 8'h00) && (O0l == 8'h00) && (Ill == 8'b00001110) && (lIl[7 : 7] == 1'b1))
                                begin
                                  OO0 <= 7;
                                  ll0 <= 1;
                                end
                              else
                                begin
                                  OO0 <= OO0;
                                  ll0 <= ll0;
                                end
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              iO0 <= 0;
            else
              if (!lli)
                iO0 <= 0;
              else
                if (ll0)
                  case (OO0)
                    0:
                      iO0 <= Iil;
                    1:
                      iO0 <= { Iil[0], lol[7 : 1] };
                    2:
                      iO0 <= { Iil[1 : 0], lol[7 : 2] };
                    3:
                      iO0 <= { Iil[2 : 0], lol[7 : 3] };
                    4:
                      iO0 <= { Iil[3 : 0], lol[7 : 4] };
                    5:
                      iO0 <= { Iil[4 : 0], lol[7 : 5] };
                    6:
                      iO0 <= { Iil[5 : 0], lol[7 : 6] };
                    7:
                      iO0 <= { Iil[6 : 0], lol[7 : 7] };
                    default:
                      iO0 <= 0;
                  endcase
                else
                  iO0 <= 0;
          end
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              I00 <= 0;
            else
              if (!lli)
                I00 <= 0;
              else
                I00 <= ll0;
          end
        assign oI0 = { iO0[0], iO0[1], iO0[2], iO0[3], iO0[4], iO0[5], iO0[6], iO0[7] };
      end
  endgenerate
  generate
    if (g_LANE_WIDTH >= 1)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                Ilo <= 0;
                llo <= 0;
                olo <= 0;
                ilo <= 0;
                loo <= 0;
                Iio <= 0;
              end
            else
              begin
                Ilo <= OI0;
                llo <= Ilo;
                olo <= llo;
                ilo <= olo;
                loo <= ((olo == 0) && (llo == 0) && (Ilo == 8'hB8)) ? 0 : ((olo == 0) && (llo == 8'hB8)) ? 1 : loo;
                Iio <= loo;
              end
          end
      end
    if (g_LANE_WIDTH >= 2)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                O0o <= 0;
                I0o <= 0;
                l0o <= 0;
                o0o <= 0;
                ooo <= 0;
                lio <= 0;
              end
            else
              begin
                O0o <= II0;
                I0o <= O0o;
                l0o <= I0o;
                o0o <= l0o;
                ooo <= ((l0o == 0) && (I0o == 0) && (O0o == 8'hB8)) ? 0 : ((l0o == 0) && (I0o == 8'hB8)) ? 1 : ooo;
                lio <= ooo;
              end
          end
      end
    if (g_LANE_WIDTH >= 3)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                i0o <= 0;
                O1o <= 0;
                I1o <= 0;
                l1o <= 0;
                ioo <= 0;
                oio <= 0;
              end
            else
              begin
                i0o <= lI0;
                O1o <= i0o;
                I1o <= O1o;
                l1o <= I1o;
                ioo <= ((I1o == 0) && (O1o == 0) && (i0o == 8'hB8)) ? 0 : ((I1o == 0) && (O1o == 8'hB8)) ? 1 : ioo;
                oio <= ioo;
              end
          end
      end
    if (g_LANE_WIDTH == 4)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              begin
                o1o <= 0;
                i1o <= 0;
                Ooo <= 0;
                Ioo <= 0;
                Oio <= 0;
                iio <= 0;
              end
            else
              begin
                o1o <= oI0;
                i1o <= o1o;
                Ooo <= i1o;
                Ioo <= Ooo;
                Oio <= ((|Ooo == 0) && (i1o == 0) && (o1o == 8'hB8)) ? 0 : ((|Ooo == 0) && (i1o == 8'hB8)) ? 1 : Oio;
                iio <= Oio;
              end
          end
      end
  endgenerate
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge CLOCK_I or
            negedge oOl)
          begin
            if (!oOl)
              IIi <= 0;
            else
              IIi <= (loo) ? 1 : 0;
          end
      end
    else
      if (g_LANE_WIDTH == 2)
        begin
          always
            @(posedge CLOCK_I or
              negedge oOl)
            begin
              if (!oOl)
                IIi <= 0;
              else
                IIi <= (loo && ooo) ? 1 : 0;
            end
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            always
              @(posedge CLOCK_I or
                negedge oOl)
              begin
                if (!oOl)
                  IIi <= 0;
                else
                  IIi <= (loo && ooo && ioo && Oio) ? 1 : 0;
              end
          end
  endgenerate
  always
    @(posedge CLOCK_I or
      negedge oOl)
    begin
      if (!oOl)
        begin
          lIi <= 0;
        end
      else
        begin
          lIi[10 : 0] <= { lIi[9 : 0], IIi };
        end
    end
  generate
    if (g_LANE_WIDTH >= 1)
      begin
        cdcfifo #(.ADDR_WIDTH(12), .DATA_WIDTH(8)) l0i(.k_lim(12'h064),
                                                       .wrrstn(loo),
                                                       .wrclk(CLOCK_I),
                                                       .wrreq(loo),
                                                       .wrdata(ilo),
                                                       .wrfull(),
                                                       .wrusedw(),
                                                       .rdrstn(loo),
                                                       .rdclk(CLOCK_I),
                                                       .rdreq(lIi[3]),
                                                       .rddata(L0_DATA_O),
                                                       .rdempty(),
                                                       .rdusedw());
      end
    if (g_LANE_WIDTH >= 2)
      begin
        cdcfifo #(.ADDR_WIDTH(12), .DATA_WIDTH(8)) o0i(.k_lim(12'h064),
                                                       .wrrstn(ooo),
                                                       .wrclk(CLOCK_I),
                                                       .wrreq(ooo),
                                                       .wrdata(o0o),
                                                       .wrfull(),
                                                       .wrusedw(),
                                                       .rdrstn(ooo),
                                                       .rdclk(CLOCK_I),
                                                       .rdreq(lIi[3]),
                                                       .rddata(L1_DATA_O),
                                                       .rdempty(),
                                                       .rdusedw());
      end
    if (g_LANE_WIDTH >= 3)
      begin
        cdcfifo #(.ADDR_WIDTH(12), .DATA_WIDTH(8)) i0i(.k_lim(12'h064),
                                                       .wrrstn(ioo),
                                                       .wrclk(CLOCK_I),
                                                       .wrreq(ioo),
                                                       .wrdata(l1o),
                                                       .wrfull(),
                                                       .wrusedw(),
                                                       .rdrstn(ioo),
                                                       .rdclk(CLOCK_I),
                                                       .rdreq(lIi[3]),
                                                       .rddata(L2_DATA_O),
                                                       .rdempty(),
                                                       .rdusedw());
      end
    if (g_LANE_WIDTH == 4)
      begin
        cdcfifo #(.ADDR_WIDTH(12), .DATA_WIDTH(8)) O1i(.k_lim(12'h064),
                                                       .wrrstn(Oio),
                                                       .wrclk(CLOCK_I),
                                                       .wrreq(Oio),
                                                       .wrdata(Ioo),
                                                       .wrfull(),
                                                       .wrusedw(),
                                                       .rdrstn(Oio),
                                                       .rdclk(CLOCK_I),
                                                       .rdreq(lIi[3]),
                                                       .rddata(L3_DATA_O),
                                                       .rdempty(),
                                                       .rdusedw());
      end
  endgenerate
endmodule
