/*****************************************************************************************************************************
--
--    File Name    : mipi_csi2_rxdecoder.v 

--    Description  : This module implements High Speed Data decoding of MIPI CSI-2 interface.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module mipi_csi2_rxdecoder #(parameter g_DATAWIDTH = 10,
                             parameter g_INPUT_DATA_INVERT = 0,
                             parameter g_LANE_WIDTH = 1,
                             parameter g_BUFF_DEPTH = 1920)
                           (input CAM_CLOCK_I,
                            input READ_CLOCK_I,
                            input RESET_n_I,
                            input RESET_READ_CLK_SYNC,
                            input wire[7 : 0] ChannelA_i,
                            input wire[7 : 0] ChannelB_i,
                            input wire[7 : 0] ChannelC_i,
                            input wire[7 : 0] ChannelD_i,
                            output wire[g_DATAWIDTH - 1 : 0] data_out_o,
                            output reg[15 : 0] word_count_o,
                            output wire pixel_valid_o,
                            output reg frame_start_o,
                            output reg frame_end_o,
                            output reg line_start_o,
                            output reg line_end_o,
                            output reg[15 : 0] frame_number_o);
  parameter I1i = 16;
  parameter l1i = 16'hb800;
  parameter o1i = 16'hb801;
  parameter i1i = (g_DATAWIDTH == 10) ? 16'hb82b : 16'hb82a;
  parameter Ooi = 0,
            Ioi = 1,
            loi = 2,
            ooi = 3,
            ioi = 4,
            Oii = 5,
            Iii = 6,
            lii = 7,
            oii = 8,
            iii = 9,
            OOOI = 10,
            IOOI = 11,
            lOOI = 12,
            oOOI = 13,
            iOOI = 14,
            OIOI = 15;
  parameter IIOI = 0,
            lIOI = 1,
            oIOI = 2,
            iIOI = 3,
            OlOI = 4,
            IlOI = 5,
            llOI = 6,
            olOI = 7;
  parameter ilOI = 0,
            O0OI = 1,
            I0OI = 2,
            l0OI = 3,
            o0OI = 4,
            i0OI = 5;
  reg[7 : 0] O1OI,
             I1OI;
  reg l1OI;
  reg o1OI;
  reg i1OI;
  reg OoOI;
  reg IoOI;
  reg[7 : 0] loOI,
             ooOI,
             ioOI,
             OiOI;
  reg[15 : 0] IiOI;
  reg[7 : 0] liOI;
  reg[7 : 0] oiOI;
  reg[g_LANE_WIDTH * 8 - 1 : 0] iiOI,
                                OOII;
  reg[3 : 0] IOII;
  reg[I1i - 1 : 0] lOII;
  reg[31 : 0] oOII;
  reg[15 : 0] iOII;
  reg[7 : 0] OIII;
  reg[1 : 0] IIII;
  reg[2 : 0] lIII;
  reg[1 : 0] oIII;
  wire iIII;
  reg OlII;
  wire IlII;
  wire llII;
  wire olII;
  wire ilII;
  wire IoII;
  wire loII;
  reg[1 : 0] O0II;
  reg ol;
  reg I0II,
      l0II;
  reg il,
      o0II;
  reg i0II,
      O1II;
  reg I1II,
      l1II;
  assign iIII = 1'b0;
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          loOI <= 8'd0;
        end
      else
        if (!iIII)
          begin
            if (g_INPUT_DATA_INVERT)
              loOI <= { ChannelA_i[0], ChannelA_i[1], ChannelA_i[2], ChannelA_i[3], ChannelA_i[4], ChannelA_i[5], ChannelA_i[6], ChannelA_i[7] };
            else
              loOI <= ChannelA_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        ooOI <= 8'd0;
      else
        if (!iIII)
          begin
            if (g_INPUT_DATA_INVERT)
              ooOI <= { ChannelB_i[0], ChannelB_i[1], ChannelB_i[2], ChannelB_i[3], ChannelB_i[4], ChannelB_i[5], ChannelB_i[6], ChannelB_i[7] };
            else
              ooOI <= ChannelB_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ioOI <= 8'd0;
        end
      else
        if (!iIII)
          begin
            if (g_INPUT_DATA_INVERT)
              ioOI <= { ChannelC_i[0], ChannelC_i[1], ChannelC_i[2], ChannelC_i[3], ChannelC_i[4], ChannelC_i[5], ChannelC_i[6], ChannelC_i[7] };
            else
              ioOI <= ChannelC_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        OiOI <= 8'd0;
      else
        if (!iIII)
          begin
            if (g_INPUT_DATA_INVERT)
              OiOI <= { ChannelD_i[0], ChannelD_i[1], ChannelD_i[2], ChannelD_i[3], ChannelD_i[4], ChannelD_i[5], ChannelD_i[6], ChannelD_i[7] };
            else
              OiOI <= ChannelD_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iiOI[7 : 0] <= 8'b0;
        end
      else
        if (g_LANE_WIDTH >= 1)
          begin
            iiOI[7 : 0] <= loOI;
          end
        else
          iiOI[7 : 0] <= 8'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iiOI[15 : 8] <= 8'b0;
        end
      else
        if (g_LANE_WIDTH >= 2)
          begin
            iiOI[15 : 8] <= { ooOI };
          end
        else
          iiOI[15 : 8] <= 8'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          iiOI[31 : 16] <= 16'b0;
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            iiOI[31 : 16] <= { OiOI, ioOI };
          end
        else
          iiOI[31 : 16] <= 16'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          lOII <= 16'b0;
        end
      else
        if (!iIII)
          begin
            lOII <= { lOII[I1i - 9 : 0], iiOI[7 : 0] };
          end
    end
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          begin
            if (!RESET_n_I)
              begin
                IOII <= 4'b0;
                l1OI <= 1'b0;
                o1OI <= 1'b0;
                i1OI <= 1'b0;
                IoOI <= 1'b0;
                IiOI <= 16'b0;
                I1OI <= 8'd0;
                O1OI <= 8'd0;
                frame_number_o <= 16'd0;
                liOI <= 8'd0;
                oiOI <= 8'd0;
                word_count_o <= 16'hFFFF;
                OOII <= 8'd0;
              end
            else
              if (!iIII)
                begin
                  case (IOII)
                    Ooi:
                      begin
                        if (lOII == l1i)
                          begin
                            l1OI <= 1;
                            IOII <= Ioi;
                          end
                        else
                          IOII <= Ooi;
                      end
                    Ioi:
                      begin
                        IOII <= loi;
                        l1OI <= 0;
                        O1OI <= lOII[7 : 0];
                      end
                    loi:
                      begin
                        IOII <= ooi;
                        l1OI <= 0;
                        I1OI <= lOII[7 : 0];
                      end
                    ooi:
                      begin
                        IOII <= ioi;
                        l1OI <= 0;
                        frame_number_o <= { I1OI, O1OI };
                      end
                    ioi:
                      begin
                        if (lOII == o1i)
                          begin
                            o1OI <= 1;
                            IOII <= lOOI;
                            i1OI <= 0;
                            l1OI <= 0;
                          end
                        else
                          if (lOII == i1i)
                            begin
                              IoOI <= 0;
                              IOII <= Oii;
                              i1OI <= 1'b1;
                            end
                          else
                            begin
                              IoOI <= 0;
                              IOII <= ioi;
                              i1OI <= 0;
                            end
                      end
                    Oii:
                      begin
                        i1OI <= 0;
                        liOI <= lOII[7 : 0];
                        IOII <= Iii;
                      end
                    Iii:
                      begin
                        i1OI <= 0;
                        oiOI <= lOII[7 : 0];
                        IOII <= lii;
                      end
                    lii:
                      begin
                        word_count_o <= { oiOI, liOI };
                        IOII <= oii;
                      end
                    oii:
                      begin
                        if (IiOI < word_count_o)
                          begin
                            IiOI <= IiOI + 1;
                            IoOI <= 1;
                            OOII <= lOII[7 : 0];
                            IOII <= oii;
                          end
                        else
                          begin
                            IiOI <= 0;
                            IoOI <= 0;
                            OOII <= 0;
                            IOII <= iii;
                          end
                      end
                    iii:
                      begin
                        IOII <= OOOI;
                      end
                    OOOI:
                      begin
                        IOII <= IOOI;
                      end
                    IOOI:
                      begin
                        IOII <= ioi;
                      end
                    lOOI:
                      begin
                        o1OI <= 0;
                        IOII <= oOOI;
                        l1OI <= 1'b0;
                      end
                    oOOI:
                      begin
                        IOII <= iOOI;
                      end
                    iOOI:
                      begin
                        IOII <= OIOI;
                      end
                    OIOI:
                      begin
                        IOII <= Ooi;
                      end
                    default:
                      IOII <= Ooi;
                  endcase
                end
          end
      end
    else
      if (g_LANE_WIDTH == 2)
        begin
          always
            @(posedge CAM_CLOCK_I or
              negedge RESET_n_I)
            begin
              if (!RESET_n_I)
                begin
                  IOII <= IIOI;
                  l1OI <= 1'b0;
                  o1OI <= 1'b0;
                  i1OI <= 1'b0;
                  OoOI <= 1'b0;
                  IoOI <= 1'b0;
                  IiOI <= 16'b0;
                  I1OI <= 8'd0;
                  O1OI <= 8'd0;
                  frame_number_o <= 16'd0;
                  liOI <= 8'd0;
                  oiOI <= 8'd0;
                  word_count_o <= 16'hFFFF;
                  OOII <= 0;
                end
              else
                case (IOII)
                  IIOI:
                    begin
                      o1OI <= 0;
                      if (iiOI == 16'hB8B8)
                        IOII <= lIOI;
                      else
                        IOII <= IIOI;
                    end
                  lIOI:
                    if (iiOI[5 : 0] == 6'h00)
                      begin
                        l1OI <= 1'b1;
                        O1OI <= iiOI[15 : 8];
                        IOII <= oIOI;
                      end
                    else
                      IOII <= IIOI;
                  oIOI:
                    begin
                      IOII <= iIOI;
                      l1OI <= 1'b0;
                      frame_number_o <= { iiOI[7 : 0], O1OI };
                    end
                  iIOI:
                    begin
                      OoOI <= 1'b0;
                      l1OI <= 0;
                      if (iiOI == 16'hB8B8)
                        IOII <= OlOI;
                      else
                        IOII <= iIOI;
                      IiOI <= 0;
                    end
                  OlOI:
                    if (iiOI[5 : 0] == 6'h01)
                      begin
                        IOII <= IIOI;
                        o1OI <= 1'b1;
                      end
                    else
                      begin
                        liOI <= iiOI[15 : 8];
                        IOII <= IlOI;
                        i1OI <= 1'b1;
                      end
                  IlOI:
                    begin
                      word_count_o <= { iiOI[7 : 0], liOI };
                      IOII <= llOI;
                      i1OI <= 1'b0;
                    end
                  llOI:
                    begin
                      i1OI <= 1'b0;
                      IiOI <= IiOI + 2;
                      IoOI <= 1;
                      OOII <= iiOI;
                      if (IiOI < (word_count_o))
                        IOII <= llOI;
                      else
                        IOII <= olOI;
                    end
                  olOI:
                    begin
                      IOII <= iIOI;
                      IoOI <= 0;
                      OoOI <= 1'b1;
                      OOII <= 0;
                    end
                  default:
                    begin
                      IOII <= IIOI;
                      l1OI <= 1'b0;
                      o1OI <= 1'b0;
                      i1OI <= 1'b0;
                      OoOI <= 1'b0;
                      IoOI <= 1'b0;
                      IiOI <= 16'b0;
                      I1OI <= 8'd0;
                      O1OI <= 8'd0;
                      frame_number_o <= 16'd0;
                      liOI <= 8'd0;
                      oiOI <= 8'd0;
                      word_count_o <= 16'hFFFF;
                      OOII <= 0;
                    end
                endcase
            end
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            always
              @(posedge CAM_CLOCK_I or
                negedge RESET_n_I)
              begin
                if (!RESET_n_I)
                  begin
                    IOII <= ilOI;
                    l1OI <= 1'b0;
                    o1OI <= 1'b0;
                    i1OI <= IoOI <= 1'b0;
                    IiOI <= 16'b0;
                    I1OI <= 8'd0;
                    O1OI <= 8'd0;
                    frame_number_o <= 16'd0;
                    liOI <= 8'd0;
                    oiOI <= 8'd0;
                    word_count_o <= 16'hFFFF;
                    OOII <= 0;
                  end
                else
                  case (IOII)
                    ilOI:
                      begin
                        o1OI <= 0;
                        if (iiOI == 32'hB8B8B8B8)
                          IOII <= O0OI;
                        else
                          IOII <= ilOI;
                      end
                    O0OI:
                      if (iiOI[5 : 0] == 6'h00)
                        begin
                          l1OI <= 1'b1;
                          frame_number_o <= iiOI[23 : 8];
                          IOII <= 4'h2;
                        end
                      else
                        IOII <= ilOI;
                    I0OI:
                      begin
                        OoOI <= 1'b0;
                        l1OI <= 0;
                        if (iiOI == 32'hB8B8B8B8)
                          IOII <= l0OI;
                        else
                          IOII <= I0OI;
                        IiOI <= 0;
                      end
                    l0OI:
                      if (iiOI[5 : 0] == 6'h01)
                        begin
                          IOII <= ilOI;
                          o1OI <= 1'b1;
                        end
                      else
                        begin
                          word_count_o <= iiOI[23 : 8];
                          IOII <= o0OI;
                          i1OI <= 1'b1;
                        end
                    o0OI:
                      begin
                        i1OI <= 1'b0;
                        IiOI <= IiOI + 4;
                        IoOI <= 1;
                        OOII <= iiOI;
                        if (IiOI < (word_count_o))
                          IOII <= o0OI;
                        else
                          IOII <= i0OI;
                      end
                    i0OI:
                      begin
                        IOII <= I0OI;
                        IoOI <= 0;
                        OoOI <= 1'b1;
                        OOII <= 0;
                      end
                    default:
                      begin
                        IOII <= ilOI;
                        l1OI <= 1'b0;
                        o1OI <= 1'b0;
                        i1OI <= 1'b0;
                        OoOI <= 1'b0;
                        IoOI <= 1'b0;
                        IiOI <= 16'b0;
                        I1OI <= 8'd0;
                        O1OI <= 8'd0;
                        frame_number_o <= 16'd0;
                        liOI <= 8'd0;
                        oiOI <= 8'd0;
                        word_count_o <= 16'hFFFF;
                        OOII <= 0;
                      end
                  endcase
              end
          end
  endgenerate
  byte2pixel_conversion #(.g_DATAWIDTH(g_DATAWIDTH), .g_LANE_WIDTH(g_LANE_WIDTH)) o1II(.CAM_CLOCK_I(CAM_CLOCK_I),
                                                                                       .READ_CLOCK_I(READ_CLOCK_I),
                                                                                       .RESET_n_I(RESET_n_I),
                                                                                       .RESET_READ_CLK_SYNC(RESET_READ_CLK_SYNC),
                                                                                       .byte_data_i(OOII),
                                                                                       .byte_en_i(IoOI),
                                                                                       .pixel_data_o(data_out_o),
                                                                                       .line_valid_o(pixel_valid_o),
                                                                                       .line_start_o(i1II),
                                                                                       .line_end_o(OoII));
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    if (!RESET_READ_CLK_SYNC)
      begin
        i0II <= 0;
        O1II <= 0;
      end
    else
      begin
        i0II <= l1OI;
        O1II <= i0II;
      end
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    if (!RESET_READ_CLK_SYNC)
      begin
        I1II <= 0;
        l1II <= 0;
      end
    else
      begin
        I1II <= o1OI;
        l1II <= I1II;
      end
  assign IoII = i0II & ~O1II;
  assign loII = I1II & ~l1II;
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_READ_CLK_SYNC))
    begin
      if (!RESET_READ_CLK_SYNC)
        begin
          frame_start_o <= 1'd0;
        end
      else
        begin
          frame_start_o <= IoII;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_READ_CLK_SYNC))
    begin
      if (!RESET_READ_CLK_SYNC)
        begin
          frame_end_o <= 1'd0;
        end
      else
        begin
          frame_end_o <= loII;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_READ_CLK_SYNC))
    begin
      if (!RESET_READ_CLK_SYNC)
        begin
          line_start_o <= 1'd0;
        end
      else
        begin
          line_start_o <= i1II;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_READ_CLK_SYNC))
    begin
      if (!RESET_READ_CLK_SYNC)
        begin
          line_end_o <= 1'd0;
        end
      else
        begin
          line_end_o <= OoII;
        end
    end
endmodule
